# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.auth.models import Group
from guardian.shortcuts import assign_perm
import pytest
# Allow everything in there to access the DB
pytestmark = pytest.mark.django_db

import logging

from nsot import exc, models
from nsot.middleware.auth import NsotObjectPermissionsBackend

from .fixtures import test_group, user, site

def test_object_level_permissions_with_ancestors(site, user, test_group):
    """Test to check object level permissions for objects with a
    ``get_ancestors`` method implementation"""
    net_8  = models.Network.objects.create(site=site, cidr=u'8.0.0.0/8')
    net_24 = models.Network.objects.create(site=site, cidr=u'8.0.0.0/24')
    net_16 = models.Network.objects.create(site=site, cidr=u'8.0.0.0/16')

    # Need to refresh the objects from the db so the updated parent_ids are
    # reflected.
    net_8.refresh_from_db()
    net_24.refresh_from_db()
    net_16.refresh_from_db()

    user.groups.add(test_group)

    check_perms = NsotObjectPermissionsBackend()
    assert check_perms.has_perm(user, 'delete_network', net_24) is False

    assign_perm('delete_network', user, net_8)
    assert check_perms.has_perm(user, 'delete_network', net_8) is True
    assert check_perms.has_perm(user, 'delete_network', net_24) is True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import macaddress.fields


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0003_auto_20150810_1751'),
    ]

    operations = [
        migrations.AlterField(
            model_name='interface',
            name='mac',
            field=macaddress.fields.MACAddressField(default=0, integer=True, null=True, db_index=True, blank=True),
        ),
        migrations.AlterField(
            model_name='interface',
            name='speed',
            field=models.IntegerField(default=1000, db_index=True),
        ),
        migrations.AlterField(
            model_name='interface',
            name='type',
            field=models.IntegerField(default=6, db_index=True, verbose_name='Interface Type', choices=[('other', 1), ('ethernet', 6), ('loopback', 24), ('tunnel', 131), ('l2vlan', 135), ('l3vlan', 136), ('mpls', 150), ('lag', 161)]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0022_auto_20151007_1847'),
    ]

    operations = [
        migrations.AlterIndexTogether(
            name='value',
            index_together=set([('resource_name', 'resource_id'), ('name', 'value', 'resource_name')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


def migrate_interface_fields(apps, schema_editor):
    """
    Migrate new Interface fields.
    """
    Interface = apps.get_model('nsot', 'Interface')
    Interface_temp = apps.get_model('nsot', 'Interface_temp')
    for ifc in Interface.objects.iterator():
        ifc_tmp = Interface_temp.objects.create(
            id=ifc.resource_ptr_id,
            name=ifc.name,
            description=ifc.description,
            device_id=ifc.device_id,
            parent_id=ifc.parent_id,
            type=ifc.type,
            speed=ifc.speed,
            mac_address=ifc.mac_address,
            site=ifc.site,
            _attributes_cache = ifc._attributes,
        )


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0017_move_network_data'),
    ]

    operations = [

        # Interface _attributes_cache, new_id
        migrations.RunPython(migrate_interface_fields),

    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import pytest
# Allow everything in there to access the DB
pytestmark = pytest.mark.django_db

from django.db import IntegrityError
from django.db.models import ProtectedError
from django.conf import settings
from django.core.exceptions import ValidationError as DjangoValidationError
import ipaddress
import logging

from nsot import exc, models

from .fixtures import admin_user, circuit, device, site, user, transactional_db


def test_creation(device):
    """Test basic Circuit creation."""
    site = device.site

    # Create a network for interface assignments
    network = models.Network.objects.create(
        cidr='10.32.0.0/24', site=site,
    )

    # A-side device/interface
    device_a = device
    iface_a = models.Interface.objects.create(
        device=device_a, name='eth0/1', addresses=['10.32.0.1/32']
    )

    # Z-side device/interface
    device_z = models.Device.objects.create(
        hostname='foo-bar2', site=site
    )
    iface_z = models.Interface.objects.create(
        device=device_z, name='eth0/1', addresses=['10.32.0.2/32']
    )

    # Create the circuit
    circuit = models.Circuit.objects.create(
        endpoint_a=iface_a, endpoint_z=iface_z
    )

    # Interface inherits endpoint_a's site
    assert circuit.site == iface_a.site

    # Name should be slugs of A/Z interfaces joined by '_'
    expected_name_t = '{endpoint_a}_{endpoint_z}'
    expected_name = expected_name_t.format(
        endpoint_a=iface_a, endpoint_z=iface_z
    )
    assert circuit.name == expected_name

    # Name slug should be the slugified version of the name
    assert circuit.name_slug == expected_name.replace('/', '_')

    # Assert property values
    assert circuit.interfaces == [iface_a, iface_z]
    assert [str(a) for a in circuit.addresses] == ['10.32.0.1/32', '10.32.0.2/32']
    assert circuit.devices == [device_a, device_z]

    # Try to create another circuit w/ the same interfaces (expecting Django
    # validation error)
    with pytest.raises(DjangoValidationError):
        c2 = models.Circuit.objects.create(
            endpoint_a=iface_a, endpoint_z=iface_z
        )

    # ... Or with A/Z sides swapped (expecting DRF validation error).
    with pytest.raises(exc.ValidationError):
        c2 = models.Circuit.objects.create(
            endpoint_a=iface_z, endpoint_z=iface_a
        )


def test_attributes(circuit):
    """Test that attributes work as expected."""
    models.Attribute.objects.create(
        site=circuit.site, resource_name='Circuit', name='cid'
    )
    models.Attribute.objects.create(
        site=circuit.site, resource_name='Circuit', name='vendor'
    )

    # Set attributes
    attrs = {'cid': 'abc123', 'vendor': 'acme'}
    circuit.set_attributes(attrs)
    assert circuit.get_attributes() == attrs

    # Test a sinmple set query just for kicks.
    query_result = models.Circuit.objects.set_query('cid=abc123 vendor=acme')
    assert list(query_result) == [circuit]

    # Verify that we can zero out attributes
    circuit.set_attributes({})
    assert circuit.get_attributes() == {}

    # And make sure no bogus attributes can be set.
    with pytest.raises(exc.ValidationError):
        circuit.set_attributes(None)

    with pytest.raises(exc.ValidationError):
        circuit.set_attributes({0: 'value'})

    with pytest.raises(exc.ValidationError):
        circuit.set_attributes({'key': 0})

    with pytest.raises(exc.ValidationError):
        circuit.set_attributes({'made_up': 'value'})


class TestInterfaceFor(object):
    @pytest.fixture
    def device_z(self, site):
        return models.Device.objects.create(site=site, hostname='foo-bar2')

    @pytest.fixture
    def interface_a(self, device):
        return models.Interface.objects.create(device=device, name='eth0')

    @pytest.fixture
    def interface_z(self, device_z):
        return models.Interface.objects.create(
            device=device_z, name='eth0')

    @pytest.fixture
    def normal_circuit(self, device_z, interface_a, interface_z):
        return models.Circuit.objects.create(
            endpoint_a=interface_a,
            endpoint_z=interface_z
        )

    @pytest.fixture
    def looped_circuit(self, device, interface_a):
        interface_z = models.Interface.objects.create(
            device=device,
            name='eth1'
        )
        return models.Circuit.objects.create(
            endpoint_a=interface_a,
            endpoint_z=interface_z,
        )

    def test_normal_conditions(self, device, device_z, interface_a,
                               interface_z, normal_circuit):
        assert normal_circuit.interface_for(device) == interface_a
        print('interface_z via circuit id = {}'.format(normal_circuit.endpoint_z.id))
        print('interface_z id = {}'.format(interface_z.id))
        assert normal_circuit.interface_for(device_z) == interface_z

    def test_single_sided(self, device, interface_a):
        """
        Make sure things don't blow up on a single-sided circuit
        """
        circuit = models.Circuit.objects.create(endpoint_a=interface_a)
        assert circuit.interface_for(device) == interface_a

    def test_looped_circuit(self, device, looped_circuit, interface_a):
        """
        Test the case when both sides of a circuit are connected to the same
        device. The method should return endpoint_a in this case.
        """
        assert looped_circuit.interface_for(device) == interface_a

    def test_bogus_device(self, device, device_z, looped_circuit):
        """
        interface_for should return None when given a device that isn't
        connected by the circuit
        """
        assert looped_circuit.interface_for(device_z) is None
        assert looped_circuit.interface_for(device) is not None



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import logging

import pytest
# Allow everything in there to access the DB
pytestmark = pytest.mark.django_db

from django.db import IntegrityError
from django.db.models import ProtectedError
from django.core.exceptions import ValidationError as DjangoValidationError
import ipaddress

from nsot import exc, models

from .fixtures import admin_user, user, site, transactional_db
from ..util import load_json


def test_reparent_bug_issues_27(site):
    """
    Test for bug described at https://github.com/dropbox/nsot/issues/27
    """

    net_8  = models.Network.objects.create(site=site, cidr=u"10.0.0.0/8")
    net_31 = models.Network.objects.create(site=site, cidr=u"10.17.244.128/31")
    net_25 = models.Network.objects.create(site=site, cidr=u"10.16.1.0/25")

    net_8.refresh_from_db()
    net_31.refresh_from_db()
    net_25.refresh_from_db()

    assert net_8.parent_id is None
    assert net_31.parent_id == net_8.id
    assert net_25.parent_id == net_8.id


def test_next_network_bug_issues_216(site):
    """
    Test for bug where Network.get_next_network() returns the wrong result.

    Ref: https://github.com/dropbox/nsot/issues/216
    """

    # Load and create the Network objects
    objects = load_json('model_tests/data/networks.json')
    [models.Network.objects.create(site=site, **n) for n in objects]

    # Get the parent
    parent = models.Network.objects.get_by_address('10.20.0.0/16')

    # We're expecting that the next /21 is going to be expected_cidr
    expected_cidr = '10.20.192.0/21'
    expected = [ipaddress.ip_network(expected_cidr)]
    next_21 = parent.get_next_network(prefix_length=21)

    assert next_21 == expected


def test_next_network_bug_issues_224(site):
    """
    Test for bug where Network.get_next_network() returns the wrong result.

    Ref: https://github.com/dropbox/nsot/issues/224
    """

    objects = load_json('model_tests/data/networks.json')
    [models.Network.objects.create(site=site, **n) for n in objects]

    # Get the parent
    parent = models.Network.objects.get_by_address('10.20.0.0/16')

    # We're expecting that the next /31 is going to be first_cidr
    first_cidr = '10.20.0.0/31'
    expected = [ipaddress.ip_network(first_cidr)]
    first_31 = parent.get_next_network(prefix_length=31)

    assert first_31 == expected

    # Create the first /31 and get the next one.
    models.Network.objects.create(cidr=first_cidr, site=site)
    next_cidr = '10.20.0.2/31'
    expected = [ipaddress.ip_network(next_cidr)]
    next_31 = parent.get_next_network(prefix_length=31)

    assert next_31 == expected


def test_next_network_bug_issues_247(site):
    """
    Test for a bug where Network.get_next-network() returns an address that is
    in the assigned state, but not any that are in the allocated state

    Ref: https://github.com/dropbox/nsot/issues/247
    """

    objects = load_json('model_tests/data/networks.json')
    [models.Network.objects.create(site=site, **n) for n in objects]

    parent = models.Network.objects.get_by_address('10.20.0.0/16')

    # Create some /32s to push our expected /32 up
    for i in range(1, 5):
        models.Network.objects.create(
            site=site,
            cidr='10.20.0.{}/32'.format(i),
            state=models.Network.ASSIGNED
        )

    expected_32 = [ipaddress.ip_network('10.20.0.5/32')]
    next_32 = parent.get_next_network(prefix_length=32)

    assert next_32 == expected_32

    expected_31 = [ipaddress.ip_network('10.20.0.6/31')]
    next_31 = parent.get_next_network(prefix_length=31)

    assert next_31 == expected_31

    expected_24 = [ipaddress.ip_network('10.20.30.0/24')]
    next_24 = parent.get_next_network(prefix_length=24)

    assert next_24 == expected_24



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import pytest

from django.db import IntegrityError
from django.db.models import ProtectedError
from django.core.exceptions import ValidationError as DjangoValidationError
import ipaddress
import json
import logging
import re

from nsot import exc, models

from .fixtures import device, user, site


# Allow everything in there to access the DB
pytestmark = pytest.mark.django_db


@pytest.fixture
def create(device, user):
    models.Change.objects.create(event='Create', obj=device, user=user)


def test_diff_device_hostname(create, device, user):
    device.hostname = 'foo-bar3'
    device.save()
    update = models.Change.objects.create(event='Update', obj=device,
                                          user=user)

    assert '-   "hostname": "foo-bar1"' in update.diff
    assert '+   "hostname": "foo-bar3"' in update.diff


def test_diff_noop(create, device, user):
    update = models.Change.objects.create(event='Update', obj=device,
                                          user=user)

    blob = json.dumps(update.resource, indent=2, sort_keys=True)

    for line_a, line_b in zip(update.diff.splitlines(), blob.splitlines()):
        assert line_a.strip() == line_b.strip()


def test_diff_delete(create, device, user):
    delete = models.Change.objects.create(event='Delete', obj=device,
                                          user=user)
    blob = json.dumps(delete.resource, indent=2, sort_keys=True)

    for line_a, line_b in zip(delete.diff.splitlines(), blob.splitlines()):
        assert line_a == '- ' + line_b



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import pytest

# Allow everything in there to access the DB
pytestmark = pytest.mark.django_db

import copy
from django.core.urlresolvers import reverse
import json
import logging
from rest_framework import status

from .fixtures import live_server, client, user, site
from .util import (
    assert_created, assert_error, assert_success, assert_deleted, load_json,
    Client, load, filter_values, get_result
)


log = logging.getLogger(__name__)


def test_filters(site, client):
    """Test field-based filters for Values."""
    # URIs
    attr_uri = site.list_uri('attribute')
    dev_uri = site.list_uri('device')
    val_uri = site.list_uri('value')

    # Pre-load the Attributes
    client.post(attr_uri, data=load('attributes.json'))

    # Populate the Device objects
    client.post(dev_uri, data=load('devices.json'))

    # Get all the Values for testing
    val_resp = client.get(val_uri)
    values = get_result(val_resp)

    # Test lookup by name
    kwargs = {'name': 'owner'}
    wanted = filter_values(values, **kwargs)
    expected = wanted
    assert_success(
        client.retrieve(val_uri, **kwargs),
        expected
    )

    # Test lookup by name + value
    kwargs = {'name': 'owner', 'value': 'jathan'}
    wanted = filter_values(values, **kwargs)
    expected = wanted
    assert_success(
        client.retrieve(val_uri, **kwargs),
        expected
    )

    # Test lookup by resource_name + resource_id
    kwargs = {'resource_name': 'Device', 'resource_id': 4}
    wanted = filter_values(values, **kwargs)
    expected = wanted
    assert_success(
        client.retrieve(val_uri, **kwargs),
        expected
    )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.deletion
import django_extensions.db.fields.json
import smart_selects.db_fields


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0020_move_value_data'),
    ]

    operations = [

        #########
        # Value #
        #########

        # Fix constraints
        migrations.AlterUniqueTogether(
            name='value',
            unique_together=set([('name', 'value', 'resource_name', 'new_resource_id')]),
        ),
        migrations.AlterIndexTogether(
            name='value',
            index_together=set([('name', 'value', 'resource_name')]),
        ),

        # Delete the resource fk
        migrations.RemoveField(
            model_name='value',
            name='resource',
        ),

        # Rename resource_id
        migrations.RenameField(
            model_name='value',
            old_name='new_resource_id',
            new_name='resource_id',
        ),

        #############
        # Resources #
        #############

        # Delete the now-legacy Resource-based multiple inheritance models.

        migrations.DeleteModel(
            name='Assignment',
        ),

        migrations.DeleteModel(
            name='Interface',
        ),

        migrations.DeleteModel(
            name='Network',
        ),

        migrations.DeleteModel(
            name='Device',
        ),

        # Delete the Resource model altogether
        migrations.DeleteModel(
            name='Resource',
        ),

        #################
        # Rename models #
        #################

        # Start renaming the new Resource-abstract concrete models.

        # Network & Network relations
        migrations.RenameModel(
            old_name='Network_temp',
            new_name='Network',
        ),
        migrations.AlterField(
            model_name='network',
            name='parent',
            field=models.ForeignKey(related_name='children', on_delete=django.db.models.deletion.PROTECT, default=None, blank=True, to='nsot.Network', null=True),
        ),

        # Device
        migrations.RenameModel(
            old_name='Device_temp',
            new_name='Device',
        ),

        # Interface & Interface relations
        migrations.RenameModel(
            old_name='Interface_temp',
            new_name='Interface',
        ),

        ##############
        # Assignment #
        ##############

        # Because Assignments is used as m2m-through on Interface, we have to
        # create the model anew.
        migrations.CreateModel(
            name='Assignment',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('address', models.ForeignKey(related_name='assignments', to='nsot.Network')),
                ('interface', models.ForeignKey(related_name='assignments', to='nsot.Interface', db_index=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
            ],
        ),

        # Interface => m2m throgh Assignment => Network
        migrations.AlterField(
            model_name='interface',
            name='addresses',
            field=models.ManyToManyField(related_name='addresses', through='nsot.Assignment', to='nsot.Network', db_index=True)
        ),

        # Ditch the intermediate temp Assignment model
        migrations.DeleteModel(
            name='Assignment_temp',
        ),

        # Interface relations
        migrations.AlterField(
            model_name='interface',
            name='device',
            field=models.ForeignKey(related_name='interfaces', verbose_name='Device', to='nsot.Device', help_text='Unique ID of the connected Device.')
        ),
        migrations.AlterField(
            model_name='interface',
            name='parent',
            field=smart_selects.db_fields.ChainedForeignKey(chained_model_field='device', related_name='children', on_delete=django.db.models.deletion.PROTECT, default=None, to='nsot.Interface', chained_field='device', blank=True, help_text='Unique ID of the parent Interface.', null=True, verbose_name='Parent'),
        ),

        ###############
        # Constraints #
        ###############

        migrations.AlterUniqueTogether(
            name='assignment',
            unique_together=set([('address', 'interface')]),
        ),
        migrations.AlterUniqueTogether(
            name='device',
            unique_together=set([('site', 'hostname')]),
        ),
        migrations.AlterUniqueTogether(
            name='network',
            unique_together=set([('site', 'ip_version', 'network_address', 'prefix_length')]),
        ),
        migrations.AlterIndexTogether(
            name='assignment',
            index_together=set([('address', 'interface')]),
        ),
        migrations.AlterIndexTogether(
            name='device',
            index_together=set([('site', 'hostname')]),
        ),
        migrations.AlterIndexTogether(
            name='network',
            index_together=set([('site', 'ip_version', 'network_address', 'prefix_length')]),
        ),


    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0026_model_field_verbose_names'),
    ]

    operations = [
        migrations.AddField(
            model_name='interface',
            name='device_hostname',
            field=models.CharField(help_text='The hostname of the Device to which the interface is bound. (Internal use only)', max_length=255, db_index=True, blank=True, editable=False),
        ),
        migrations.AlterIndexTogether(
            name='interface',
            index_together=set([('device_hostname', 'name'), ('device', 'name')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0032_add_indicies_to_change'),
    ]

    operations = [
        migrations.AddField(
            model_name='interface',
            name='name_slug',
            field=models.CharField(null=True, editable=False, max_length=255, help_text='Slugified version of the name field, used for the natural key', unique=True, db_index=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import pytest
from nsot import exc, models
from .fixtures import circuit, site


# Allow everything in there to access the DB
pytestmark = pytest.mark.django_db


@pytest.fixture
def asn_attribute(site):
    return models.Attribute.objects.create(
        site=site,
        name='asn',
        resource_name='Protocol',
    )


@pytest.fixture
def bgp(asn_attribute, site):
    bgp = models.ProtocolType.objects.create(
        name='bgp',
        description='Border Gateway Protocol',
        site=site,
    )
    bgp.required_attributes.add(asn_attribute)

    return bgp


@pytest.fixture
def area_attribute(site):
    return models.Attribute.objects.create(
        site=site,
        name='area',
        resource_name='Protocol',
    )


@pytest.fixture
def ospf(area_attribute, site):
    ospf = models.ProtocolType.objects.create(
        name='ospf',
        description='Open Shortest Path First protocol',
        site=site
    )
    ospf.required_attributes.add(area_attribute)

    return ospf


class ProtocolTestCase(object):
    """Shared fixtures for Protocol(Type) tests."""
    @pytest.fixture
    def device(self, circuit):
        return circuit.endpoint_a.device

    @pytest.fixture
    def interface(self, circuit):
        return circuit.endpoint_a


class TestProtocolType(ProtocolTestCase):
    """Test ProtocolType constraints."""
    def test_required_attributes_site(self, site, bgp, area_attribute):
        """Test that required_attributes are in same site."""
        # Create a 2nd site and 2 protocol attributes.
        site2 = models.Site.objects.create(name='Site2')
        attr1 = models.Attribute.objects.create(
            resource_name='Protocol', name='attr1', site=site2
        )
        attr2 = models.Attribute.objects.create(
            resource_name='Protocol', name='attr2', site=site2
        )

        # Adding attributes from wrong site should fail.
        with pytest.raises(exc.ValidationError):
            bgp.required_attributes.add(attr1, attr2)

    def test_required_attributes_resource(self, site, bgp):
        """Test Protocol attributes only for required_attributes."""
        bad_attr = models.Attribute.objects.create(
            site=site, name='bad', resource_name='Device'
        )

        # Adding attributes from wrong resource should fail.
        with pytest.raises(exc.ValidationError):
            bgp.required_attributes.add(bad_attr)


class TestCreation(ProtocolTestCase):
    def test_normal_conditions(self, device, circuit, bgp):
        """
        Under normal conditions, ensure that a Protocol can be created without
        raising any exceptions
        """
        protocol = models.Protocol.objects.create(
            device=device,
            circuit=circuit,
            type=bgp,
            description='My fancy peer',
            attributes={
                'asn': '1234',
            }
        )
        protocol.save()

        assert protocol.site_id == device.site_id
        assert protocol in device.protocols.all()
        assert protocol in circuit.protocols.all()

    def test_bad_circuit(self, device, circuit, bgp):
        """
        Ensure a ValidationError is thrown when we try to create a Protocol
        where the Circuit is not attached to the Device
        """
        bad_circuit = circuit
        bad_circuit.endpoint_a = bad_circuit.endpoint_z

        with pytest.raises(exc.ValidationError):
            models.Protocol.objects.create(
                device=device,
                circuit=bad_circuit,
                type=bgp,
                attributes={
                    'asn': '1234',
                }
            )

    def test_attributes(self, device, circuit, bgp):
        """ Ensure that we can set arbitrary attributes on a Protocol """
        models.Attribute.objects.create(
            site=device.site, resource_name='Protocol', name='import_policies')

        protocol = models.Protocol.objects.create(
            device=device,
            circuit=circuit,
            type=bgp,
            attributes={
                'asn': '1234',
                'import_policies': 'foo'
            }
        )

        assert protocol.get_attributes()['import_policies'] == 'foo'

    def test_missing_attributes(self, device, circuit, bgp):
        """
        Test that a ValidationError is thrown when we try to create a Protocol.
        The BGP ProtocolType we defined has a required attribute `asn`
        """
        with pytest.raises(exc.ValidationError):
            models.Protocol.objects.create(
                device=device,
                circuit=circuit,
                type=bgp,
                attributes={},
            )

    def test_another_protocol(self, device, interface, ospf):
        """
        Test that we can add a Protocol with second ProtocolType and that it
        doesn't raise any sort of exception
        """
        models.Protocol.objects.create(
            device=device,
            interface=interface,
            type=ospf,
            attributes={'area': 'threeve'}
        )

    def test_mixed_attrs(self, device, circuit, bgp, area_attribute):
        """
        Test that we can add a Protocol of one ProtocolType with an attribute
        that's required by another ProtocolType without anything weird
        happening
        """
        protocol = models.Protocol.objects.create(
            device=device,
            circuit=circuit,
            type=bgp,
            attributes={
                'asn': '1234',
                'area': 'threeve',
            }
        )

        assert protocol.get_attributes()['asn'] == '1234'
        assert protocol.get_attributes()['area'] == 'threeve'


class TestUnicode(ProtocolTestCase):
    """
    Tests for the __unicode__ method on Protocol
    """
    @pytest.fixture
    def base_protocol(self, bgp, circuit):
        device = circuit.endpoint_a.device

        return models.Protocol.objects.create(
            device=device,
            type=bgp,
            attributes={
                'asn': '1234',
            }
        )

    def test_circuit(self, base_protocol, circuit):
        """
        Case when only a circuit is set on the Protocol (no interface)
        """
        expected = 'bgp over foo-bar1:eth0_foo-bar2:eth0'

        base_protocol.circuit = circuit
        base_protocol.save()

        assert base_protocol.circuit is not None
        assert base_protocol.interface is None

        assert unicode(base_protocol) == expected

    def test_interface(self, base_protocol, interface):
        """
        Case when only an interface is set on the Protocol (no circuit)
        """
        expected = 'bgp on foo-bar1:eth0'

        base_protocol.interface = interface
        base_protocol.save()

        assert base_protocol.interface is not None
        assert base_protocol.circuit is None

        assert unicode(base_protocol) == expected

    def test_neither_circuit_nor_interface(self, base_protocol):
        """
        Case when neither a circuit nor interface are set
        """
        expected = 'bgp on foo-bar1'

        assert base_protocol.circuit is None
        assert base_protocol.interface is None

        assert unicode(base_protocol) == expected

    def test_both_circuit_and_interface(self, base_protocol, circuit,
                                        interface):
        """
        Case when both a circuit and interface are set
        """
        expected = 'bgp over foo-bar1:eth0_foo-bar2:eth0'

        base_protocol.circuit = circuit
        base_protocol.interface = interface
        base_protocol.save()

        assert base_protocol.circuit is not None
        assert base_protocol.interface is not None

        assert unicode(base_protocol) == expected



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import pytest

# Allow everything in there to access the DB
pytestmark = pytest.mark.django_db

import copy
from django.core.urlresolvers import reverse
import json
import logging
from rest_framework import status

from .fixtures import live_server, client, user, site, user_client
from .util import (
    assert_created, assert_error, assert_success, assert_deleted, load_json,
    Client, TestSite, load, filter_circuits, get_result
)


log = logging.getLogger(__name__)


@pytest.fixture
def asn_attribute(site, client):
    """``Protocol:asn`` attribute."""
    attr_uri = site.list_uri('attribute')
    resp = client.create(attr_uri, name='asn', resource_name='Protocol')
    return resp.json()


@pytest.fixture
def metric_attribute(site, client):
    """``Protocol:metric`` attribute."""
    attr_uri = site.list_uri('attribute')
    resp = client.create(attr_uri, name='metric', resource_name='Protocol')
    return resp.json()


@pytest.fixture
def bgp_type(site, client, asn_attribute):
    """ProtocolType instance of 'bgp' with 'asn' as required attribute."""
    pt_uri = site.list_uri('protocoltype')
    resp = client.create(pt_uri, name='bgp', required_attributes=['asn'])
    return resp.json()


@pytest.fixture
def isis_type(site, client, metric_attribute):
    """ProtocolType instance of 'isis' with 'metric' as required attribute."""
    pt_uri = site.list_uri('protocoltype')
    resp = client.create(pt_uri, name='isis', required_attributes=['metric'])
    return resp.json()


class TestProtocolType(object):
    """Tests for ProtocolType resource object."""

    def test_name_uniqueness(self, site, client, bgp_type):
        """Test that site/name uniqueness is enforced."""
        pt_uri = site.list_uri('protocoltype')

        # BGP already exists in this site.
        assert_error(
            client.create(pt_uri, name='bgp'),
            status.HTTP_400_BAD_REQUEST
        )

    def test_lookup_by_natural_key(self, site, client, bgp_type):
        """Test that ProtocolType can be looked up by natural key."""
        natural_uri = site.detail_uri('protocoltype', id='bgp')
        assert_success(client.retrieve(natural_uri), bgp_type)

    def test_required_attributes_site(self, site, client, bgp_type):
        """Test required_attributes are in same site."""
        site_uri = reverse('site-list')
        site_resp = client.create(site_uri, name='Other Site')
        site2 = TestSite(site_resp.json())

        # Create site2 attribute/protocoltype
        attr_uri = site2.list_uri('attribute')
        pt_uri = site2.list_uri('protocoltype')

        # Attribute
        attr_resp = client.create(attr_uri, name='bad', resource_name='Device')
        attr = attr_resp.json()

        # ProtocolType
        pt_resp = client.create(pt_uri, name='bgp2')
        pt = pt_resp.json()
        pt_obj_uri = site2.detail_uri('protocoltype', id=pt['id'])

        # Should fail.
        pt['required_attributes'] = [attr['id']]
        assert_error(
            client.update(pt_obj_uri, **pt),
            status.HTTP_400_BAD_REQUEST
        )

    def test_required_attributes_resource(self, site, client, bgp_type,
                                          asn_attribute):
        """Test Protocol attributes only for required_attributes."""
        attr_uri = site.list_uri('attribute')
        pt_uri = site.list_uri('protocoltype')

        r = client.create(attr_uri, name='bad', resource_name='Device')
        attr = r.json()
        bad_attributes = [attr['id']]

        pt_resp = client.create(pt_uri, name='bgp2')
        pt = pt_resp.json()
        pt_obj_uri = site.detail_uri('protocoltype', id=pt['id'])

        # Set bad required_attributes. Should fail.
        pt['required_attributes'] = bad_attributes
        assert_error(
            client.update(pt_obj_uri, **pt),
            status.HTTP_400_BAD_REQUEST
        )

    def test_update_required_attributes(self, site, client, bgp_type,
                                        asn_attribute):
        """Test update of required_attributes by id or natural key."""
        pt_obj_uri = site.detail_uri('protocoltype', id=bgp_type['id'])
        attrs_by_id = [asn_attribute['id']]
        attrs_by_name = [asn_attribute['name']]

        payload = copy.deepcopy(bgp_type)
        expected = copy.deepcopy(bgp_type)

        # Set required_attributes by id. Should succeed.
        payload['required_attributes'] = attrs_by_id
        expected['required_attributes'] = attrs_by_name
        assert_success(client.update(pt_obj_uri, **payload), expected)

        # Set required_attributes by name. Should succeed.
        payload['required_attributes'] = attrs_by_name
        assert_success(client.update(pt_obj_uri, **payload), expected)

        # Set bogus required_attributes. Should fail.
        payload['required_attributes'] = ['bogus']
        assert_error(
            client.update(pt_obj_uri, **payload),
            status.HTTP_400_BAD_REQUEST
        )

    def test_update_by_natural_key(self, site, client, bgp_type):
        """Test updated of ProtocolType using natural key."""
        natural_uri = site.detail_uri('protocoltype', id=bgp_type['name'])
        bgp_type['required_attributes'] = []
        assert_success(client.update(natural_uri, **bgp_type), bgp_type)

    def test_partial_update(self, site, client, bgp_type):
        """Test partial update of ProtocolType."""
        obj_uri = site.detail_uri('protocoltype', id=bgp_type['id'])

        # Partial update to name
        bgp_type['name'] = 'patched'
        assert_success(
            client.partial_update(obj_uri, name='patched'),
            bgp_type
        )

        # Partial update to description
        bgp_type['description'] = 'also patched'
        assert_success(
            client.partial_update(obj_uri, description='also patched'),
            bgp_type
        )

    def test_delete_by_id(self, site, client, bgp_type):
        """Test delete by primary key."""
        id_uri = site.detail_uri('protocoltype', id=bgp_type['id'])
        assert_deleted(client.delete(id_uri))

    def test_delete_by_natural_key(self, site, client, bgp_type):
        """Test delete by natural key."""
        natural_uri = site.detail_uri('protocoltype', id=bgp_type['name'])
        assert_deleted(client.delete(natural_uri))

    def test_filtering(self, site, client, bgp_type):
        """Test query parameter filtering."""
        pt_uri = site.list_uri('protocoltype')
        bgp2_resp = client.create(pt_uri, name='bgp2', description='bgp2')
        bgp2_type = get_result(bgp2_resp)

        # Filter by name
        assert_success(client.retrieve(pt_uri, name='bgp'), [bgp_type])

        # Filter by description
        assert_success(client.retrieve(pt_uri, description='bgp2'), [bgp2_type])


class ProtocolTestCase(object):
    """Reusable test class for Protocol objects with common fixtures."""

    @pytest.fixture
    def device(self, site, client):
        """foo-bar1"""
        dev_uri = site.list_uri('device')
        r = client.create(dev_uri, hostname='foo-bar1')
        dev = get_result(r)
        return dev

    @pytest.fixture
    def device2(self, site, client):
        """spam-bar2"""
        dev_uri = site.list_uri('device')
        r = client.create(dev_uri, hostname='spam-bar2')
        dev = get_result(r)
        return dev

    @pytest.fixture
    def interface(self, site, client, device):
        """foo-bar1:eth0"""
        ifc_uri = site.list_uri('interface')
        r = client.create(ifc_uri, device=device['id'], name='eth0')
        ifc = get_result(r)
        return ifc

    @pytest.fixture
    def circuit(self, site, client, device, interface):
        cir_uri = site.list_uri('circuit')
        dev_uri = site.list_uri('device')
        ifc_uri = site.list_uri('interface')
        net_uri = site.list_uri('network')

        # Devices
        dev_a = device
        dev_z_resp = client.create(dev_uri, hostname='foo-bar2')
        dev_z = get_result(dev_z_resp)

        # Parent network for interface assignments
        net_resp = client.create(net_uri, cidr='10.32.0.0/24')
        net = get_result(net_resp)

        # Interfaces
        if_a = interface
        if_z_resp = client.create(
          ifc_uri, device=dev_z['id'], name='eth0', addresses=['10.32.0.2/32']
        )
        if_z = get_result(if_z_resp)

        cir_resp = client.create(
          cir_uri, endpoint_a=if_a['id'], endpoint_z=if_z['id']
        )
        cir = get_result(cir_resp)

        return cir

    @pytest.fixture
    def bgp_protocol(self, site, client, device, bgp_type, circuit):
        """Protocol w/ circuit."""
        proto_uri = site.list_uri('protocol')
        proto_resp = client.create(
            proto_uri, device=device['hostname'], type=bgp_type['name'],
            attributes={'asn': '12345'}, circuit=circuit['name_slug'],
            description='Border Gateway Protocol'
        )
        proto = get_result(proto_resp)
        return proto

    @pytest.fixture
    def isis_protocol(self, site, client, device, isis_type, interface):
        """Protocol w/ interface."""
        proto_uri = site.list_uri('protocol')
        proto_resp = client.create(
            proto_uri, device=device['hostname'], type=isis_type['name'],
            attributes={'metric': '100'}, interface=interface['name_slug'],
            description='IS-IS'
        )
        proto = get_result(proto_resp)
        return proto


class TestCreation(ProtocolTestCase):
    def test_basic(self, site, client, device, bgp_type):
        """Test basic creation of a Protocol."""
        proto_uri = site.list_uri('protocol')

        # - Device referenced by natural key
        # - Type referenced by natural key
        proto_resp = client.create(
            proto_uri, device=device['hostname'], type=bgp_type['name'],
            attributes={'asn': '12345'}
        )
        proto = get_result(proto_resp)
        proto_obj_uri = site.detail_uri('protocol', id=proto['id'])

        # Valid creation
        assert_created(proto_resp, proto_obj_uri)

        # Missing required attributes (per .type). Will fail.
        assert_error(
            client.create(
                proto_uri, device=device['hostname'], type=bgp_type['name']
            ),
            status.HTTP_400_BAD_REQUEST
        )

    def test_with_interface(self, site, client, device, interface, bgp_type):
        """Test creation of a Protocol with a bound Interface."""
        proto_uri = site.list_uri('protocol')

        # - Device referenced by natural key
        # - Type referenced by natural key
        # - Interface referenced by natural key
        proto_resp = client.create(
            proto_uri, device=device['hostname'], type=bgp_type['name'],
            interface=interface['name_slug'], attributes={'asn': '12345'}
        )
        proto = get_result(proto_resp)
        proto_obj_uri = site.detail_uri('protocol', id=proto['id'])

        assert_created(proto_resp, proto_obj_uri)

    def test_with_circuit(self, site, client, device, circuit, bgp_type):
        """Test creation of a Protocol with a bound Interface."""
        proto_uri = site.list_uri('protocol')

        # - Device referenced by natural key
        # - Type referenced by natural key
        # - Circuit referenced by natural key
        proto_resp = client.create(
            proto_uri, device=device['hostname'], type=bgp_type['name'],
            circuit=circuit['name_slug'], attributes={'asn': '12345'}
        )
        proto = get_result(proto_resp)
        proto_obj_uri = site.detail_uri('protocol', id=proto['id'])

        assert_created(proto_resp, proto_obj_uri)


class TestRetrieval(ProtocolTestCase):
    def test_lookup_by_id(self, site, client, bgp_protocol):
        """Test basic lookup by primary key."""
        obj_uri = site.detail_uri('protocol', id=bgp_protocol['id'])
        assert_success(client.retrieve(obj_uri), bgp_protocol)

    def test_filters_by_id(self, site, client, device, interface,
                           circuit, bgp_type, bgp_protocol, isis_protocol):
        """Test filtering by primary key (id)."""
        proto_uri = site.list_uri('protocol')

        # Filter by description
        expected = [isis_protocol]
        assert_success(
            client.retrieve(proto_uri, description='IS-IS'),
            expected
        )

        # Filter by type
        expected = [bgp_protocol]
        assert_success(
            client.retrieve(proto_uri, type=bgp_type['id']),
            expected
        )

        # Filter by device
        expected = [bgp_protocol, isis_protocol]
        assert_success(
            client.retrieve(proto_uri, device=device['id']),
            expected
        )

        # Filter by circuit
        expected = [bgp_protocol]
        assert_success(
            client.retrieve(proto_uri, circuit=circuit['id']),
            expected
        )

        # Filter by interface
        expected = [isis_protocol]
        assert_success(
            client.retrieve(proto_uri, interface=interface['id']),
            expected
        )

    def test_filters_by_natural_key(self, site, client, device, interface,
                                    circuit, bgp_protocol, isis_protocol):
        """Test filtering by natural key."""
        proto_uri = site.list_uri('protocol')

        # Filter by description
        expected = [isis_protocol]
        assert_success(
            client.retrieve(proto_uri, description='IS-IS'),
            expected
        )

        # Filter by type
        expected = [bgp_protocol]
        assert_success(client.retrieve(proto_uri, type='bgp'), expected)

        # Filter by device
        expected = [bgp_protocol, isis_protocol]
        assert_success(
            client.retrieve(proto_uri, device=device['hostname']),
            expected
        )

        # Filter by circuit
        expected = [bgp_protocol]
        assert_success(
            client.retrieve(proto_uri, circuit=circuit['name_slug']),
            expected
        )

        # Filter by interface
        expected = [isis_protocol]
        assert_success(
            client.retrieve(proto_uri, interface=interface['name_slug']),
            expected
        )

    def test_set_queries(self, site, client, bgp_protocol, isis_protocol):
        """Test set query operations."""
        proto_uri = site.list_uri('protocol')
        query_uri = site.query_uri('protocol')
        attr_uri = site.list_uri('attribute')

        bgp = bgp_protocol
        isis = isis_protocol

        bgp_uri = site.detail_uri('protocol', id=bgp['id'])
        isis_uri = site.detail_uri('protocol', id=isis['id'])

        bgp['attributes'].update({'admin_status': 'down', 'peer_as': '19679'})
        isis['attributes'].update({'admin_status': 'up', 'peer_as': '19679'})

        # Pre-load the Attributes
        client.post(attr_uri, data=load('attributes.json'))

        # Update the Protocol objects w/ thew new attributes
        client.update(bgp_uri, **bgp)
        client.update(isis_uri, **isis)

        # Set queries
        # INTERSECTION: peer_as=19679
        expected = [bgp, isis]
        assert_success(
            client.retrieve(query_uri, query='peer_as=19679'),
            expected
        )

        # INTERSECTION: peer_as=19679 asn=12345
        expected = [bgp]
        assert_success(
            client.retrieve(query_uri, query='peer_as=19679 asn=12345'),
            expected
        )

        # DIFFERENCE: -asn=12345
        expected = [isis]
        assert_success(
            client.retrieve(query_uri, query='-asn=12345'),
            expected
        )

        # UNION: asn=12345 +metric=100
        expected = [bgp, isis]
        assert_success(
            client.retrieve(query_uri, query='asn=12345 +metric=100'),
            expected
        )

        # UNIQUE: peer_as=19679 admin_status=up
        expected = [isis]
        assert_success(
            client.retrieve(
                query_uri, query='peer_as=19679 admin_status=up', unique=True
            ),
            expected
        )

        # ERROR: not unique (peer_as=19679)
        assert_error(
            client.retrieve(query_uri, query='peer_as=19679', unique=True),
            status.HTTP_400_BAD_REQUEST
        )

        # ERROR: no result (vendor=bogus)
        assert_error(
            client.retrieve(query_uri, query='peer_as=tacos', unique=True),
            status.HTTP_400_BAD_REQUEST
        )

        # ERROR: bad query
        assert_error(
            client.retrieve(query_uri, query='bacon=delicious'),
            status.HTTP_400_BAD_REQUEST
        )


class TestUpdate(ProtocolTestCase):
    """Test update on Protocol objects."""
    def test_update_by_id(self, site, client, bgp_protocol, device):
        obj_uri = site.detail_uri('protocol', id=bgp_protocol['id'])
        expected = copy.deepcopy(bgp_protocol)
        bgp_protocol['device'] = device['id']
        assert_success(client.update(obj_uri, **bgp_protocol), expected)

    def test_update_related_natural_key(self, site, client, bgp_protocol,
                                        isis_protocol, isis_type, device,
                                        device2, interface):
        """Test update of related fields using natural keys."""
        obj_uri = site.detail_uri('protocol', id=bgp_protocol['id'])

        expected = copy.deepcopy(bgp_protocol)

        # Update type
        expected['type'] = isis_type['name']
        expected['attributes'].update({'metric': '100'})
        assert_success(client.update(obj_uri, **expected), expected)

        # Update device
        circuit_name = expected['circuit']
        expected['type'] = 'bgp'
        expected['device'] = device2['hostname']
        expected['circuit'] = None
        assert_success(client.update(obj_uri, **expected), expected)

        # Interface (Flip interface w/ circuit)
        expected['device'] = device['hostname']
        expected['circuit'] = None
        expected['interface'] = interface['name_slug']
        assert_success(client.update(obj_uri, **expected), expected)

        # Circuit (Flip interface w/ circuit)
        expected['circuit'] = circuit_name
        expected['interface'] = None
        assert_success(client.update(obj_uri, **expected), expected)

    def test_partial_update(self, site, client, bgp_protocol,
                                        isis_protocol, isis_type, device,
                                        device2, interface):
        """Test partial update for each field."""
        obj_uri = site.detail_uri('protocol', id=bgp_protocol['id'])

        expected = copy.deepcopy(bgp_protocol)

        # description
        payload = {'description': 'patched'}
        expected.update(payload)
        assert_success(client.partial_update(obj_uri, **payload), expected)

        # auth_string
        payload = {'auth_string': 'abc123'}
        expected.update(payload)
        assert_success(client.partial_update(obj_uri, **payload), expected)

        # device
        payload = {'device': device['hostname']}
        expected.update(payload)
        assert_success(client.partial_update(obj_uri, **payload), expected)

        # circuit
        payload = {'circuit': None}
        expected.update(payload)
        assert_success(client.partial_update(obj_uri, **payload), expected)

        # interface
        payload = {'interface': None}
        expected.update(payload)
        assert_success(client.partial_update(obj_uri, **payload), expected)

        # type
        payload = {'type': isis_type['name'], 'attributes': {'metric': '100'}}
        expected.update(payload)
        assert_success(client.partial_update(obj_uri, **payload), expected)

        # attributes
        payload = {'attributes': {'metric': '200'}}
        expected.update(payload)
        assert_success(client.partial_update(obj_uri, **payload), expected)


class TestDeletion(ProtocolTestCase):
    """Test delete on Protocol objects."""
    def test_delete_by_id(self, site, client, bgp_protocol):
        """Test simple delete by primary key."""
        obj_uri = site.detail_uri('protocol', id=bgp_protocol['id'])
        assert_deleted(client.delete(obj_uri))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models

from nsot.util import slugify_interface


def add_name_slug(apps, schema_editor):
    """Correctly name_slug for every Interface with slash in the name."""
    Interface = apps.get_model('nsot', 'Interface')
    for i in Interface.objects.iterator():
        name_slug = slugify_interface(
            device_hostname=i.device_hostname, name=i.name
        )
        i.name_slug = name_slug
        i.save()


def remove_name_slug(apps, schema_editor):
    Interface = apps.get_model('nsot', 'Interface')
    Interface.objects.update(name_slug=None)


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0034_populate_interface_name_slug'),
    ]

    operations = [
        migrations.RunPython(add_name_slug, remove_name_slug)
    ]



from django.core.urlresolvers import reverse
import json
import logging
import os
import pytest
from pytest_django.fixtures import live_server, django_user_model, settings
import requests

from .util import Client, TestSite


log = logging.getLogger(__name__)


# API version to use for the API client
API_VERSION = os.getenv('NSOT_API_VERSION')


@pytest.fixture
def user(django_user_model):
    """Create and return a non-admin user."""
    user = django_user_model.objects.create(email='user@localhost')
    return user


@pytest.fixture
def site(live_server):
    client = Client(live_server)
    site_uri = reverse('site-list')  # /api/sites/
    resp = client.create(site_uri, name='Test Site')

    site = TestSite(resp.json())
    return site


@pytest.fixture
def client(live_server):
    """Create and return an admin client."""
    return Client(live_server, api_version=API_VERSION)


@pytest.fixture
def user_client(live_server):
    """Create and return a non-admin client."""
    return Client(live_server, user='user', api_version=API_VERSION)


@pytest.fixture
def nosuperuser_settings(settings):
    """Return settings that have default superuser users disabled."""
    settings.NSOT_NEW_USERS_AS_SUPERUSER = False
    return settings



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields.json


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0028_populate_interface_device_hostname'),
    ]

    operations = [
        migrations.CreateModel(
            name='Circuit',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('_attributes_cache', django_extensions.db.fields.json.JSONField(help_text='Local cache of attributes. (Internal use only)', blank=True)),
                ('name', models.CharField(default='', help_text="Unique display name of the Circuit. If not provided, defaults to '{device_a}:{interface_a}_{device_z}:{interface_z}'", unique=True, max_length=255)),
                ('endpoint_a', models.OneToOneField(related_name='circuit_a', on_delete=django.db.models.deletion.PROTECT, verbose_name='A-side endpoint Interface', to='nsot.Interface', help_text='Unique ID of Interface at the A-side.')),
                ('site', models.ForeignKey(related_name='circuits', on_delete=django.db.models.deletion.PROTECT, to='nsot.Site', help_text='Unique ID of the Site this Circuit is under.')),
                ('endpoint_z', models.OneToOneField(related_name='circuit_z', null=True, on_delete=django.db.models.deletion.PROTECT, to='nsot.Interface', help_text='Unique ID of Interface at the Z-side.', verbose_name='Z-side endpoint Interface')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AlterField(
            model_name='attribute',
            name='resource_name',
            field=models.CharField(help_text='The name of the Resource to which this Attribute is bound.', max_length=20, verbose_name='Resource Name', db_index=True, choices=[('Device', 'Device'), ('Interface', 'Interface'), ('Network', 'Network'), ('Circuit', 'Circuit')]),
        ),
        migrations.AlterField(
            model_name='change',
            name='resource_name',
            field=models.CharField(help_text='The name of the Resource for this Change.', max_length=20, verbose_name='Resource Type', db_index=True, choices=[('Network', 'Network'), ('Attribute', 'Attribute'), ('Site', 'Site'), ('Interface', 'Interface'), ('Circuit', 'Circuit'), ('Device', 'Device')]),
        ),
        migrations.AlterField(
            model_name='value',
            name='resource_name',
            field=models.CharField(help_text='The name of the Resource type to which the Value is bound.', max_length=20, verbose_name='Resource Type', db_index=True, choices=[('Network', 'Network'), ('Attribute', 'Attribute'), ('Site', 'Site'), ('Interface', 'Interface'), ('Circuit', 'Circuit'), ('Device', 'Device')]),
        ),
    ]



"""
Django settings for nsot project using Django 1.8.
"""


# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import macaddress
from netaddr import eui
import os
import re
import sys

# Path where the code is found. (aka project root)
BASE_DIR = os.path.normpath(os.path.join(os.path.dirname(__file__), os.pardir))

# Path where the config file is found.
CONF_ROOT = os.path.abspath(os.path.dirname(__file__))

# A boolean that turns on/off debug mode. Never deploy a site into production
# with DEBUG turned on.
# Default: False
DEBUG = False

#################
# Core Settings #
#################

# A tuple of strings designating all applications that are enabled in this
# Django installation.
INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django_extensions',
    'django_filters',
    'guardian',
    'smart_selects',
    'rest_framework',
    'rest_framework_swagger',
    'custom_user',
    'nsot',
)

# The model to use to represent a User.
AUTH_USER_MODEL = 'nsot.User'
ANONYMOUS_USER_NAME = 'anonymous@service.local'

# A tuple of authentication backend classes (as strings) to use when attempting
# to authenticate a user.
# https://docs.djangoproject.com/en/1.8/topics/auth/customizing/#authentication-backends
AUTHENTICATION_BACKENDS = (
    'nsot.middleware.auth.EmailHeaderBackend',
    'django.contrib.auth.backends.ModelBackend',
    'nsot.middleware.auth.NsotObjectPermissionsBackend',
)

# A tuple of middleware classes to use.
# https://docs.djangoproject.com/en/1.8/topics/http/middleware/
MIDDLEWARE_CLASSES = (
    'nsot.middleware.request_logging.LoggingMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'nsot.middleware.auth.EmailHeaderMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.security.SecurityMiddleware',
)

# A string representing the full Python import path to your root URLconf.
ROOT_URLCONF = 'nsot.conf.urls'

# The full Python path of the WSGI application object that Django's built-in
# servers (e.g. runserver) will use.
WSGI_APPLICATION = 'nsot.wsgi.application'

# When set to True, if the request URL does not match any of the patterns in
# the URLconf and it doesn't end in a slash, an HTTP redirect is issued to the
# same URL with a slash appended. Note that the redirect may cause any data
# submitted in a POST request to be lost.
# Default: True
APPEND_SLASH = True

from nsot.version import __version__
NSOT_VERSION = __version__

# Template loaders. The NSoT web UI is written in Angular.js using Jinja2
# templates.
TEMPLATES = [
    {
        "BACKEND": "django_jinja.backend.Jinja2",
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.csrf",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.request",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
            ],
            "extensions": [
                "jinja2.ext.do",
                "jinja2.ext.loopcontrols",
                "jinja2.ext.with_",
                "jinja2.ext.i18n",
                "jinja2.ext.autoescape",
                "django_jinja.builtins.extensions.CsrfExtension",
                "django_jinja.builtins.extensions.CacheExtension",
                "django_jinja.builtins.extensions.TimezoneExtension",
                "django_jinja.builtins.extensions.UrlsExtension",
                "django_jinja.builtins.extensions.StaticFilesExtension",
                "django_jinja.builtins.extensions.DjangoFiltersExtension",
            ],
        }
    },
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            os.path.join(BASE_DIR, 'templates').replace('\\', '/'),
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                "django.template.context_processors.csrf",
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'nsot.ui.context_processors.app_version',
            ],
        },
    },
]

##################
# REST Framework #
##################
# Release version of the API to publish.
NSOT_API_VERSION = '1.0'

# Settings for Django REST Framework (DRF)
REST_FRAMEWORK = {
    'DEFAULT_FILTER_BACKENDS': (
        'django_filters.rest_framework.DjangoFilterBackend',
    ),
    'DEFAULT_RENDERER_CLASSES': [
        'rest_framework.renderers.JSONRenderer',
        'nsot.api.renderers.FilterlessBrowsableAPIRenderer',
    ],
    'DEFAULT_PAGINATION_CLASS': 'rest_framework.pagination.LimitOffsetPagination',
    'DEFAULT_VERSIONING_CLASS': 'rest_framework.versioning.AcceptHeaderVersioning',
    'DEFAULT_VERSION': NSOT_API_VERSION,
    'DEFAULT_PERMISSION_CLASSES': (
        'rest_framework.permissions.DjangoModelPermissions',
    ),
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'nsot.api.auth.AuthTokenAuthentication',
        'nsot.api.auth.EmailHeaderAuthentication',
        'rest_framework.authentication.BasicAuthentication',
        'rest_framework.authentication.SessionAuthentication',
    ),
    'EXCEPTION_HANDLER': 'nsot.exc.custom_exception_handler',
    'TEST_REQUEST_DEFAULT_FORMAT': 'json',
    'PAGE_SIZE': None,  # No default pagination.
}

############
# Database #
############
# https://docs.djangoproject.com/en/1.8/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'nsot.sqlite3'),
    }
}

#########
# Cache #
#########
# NSoT includes supportfor caching of API results, however the default is to
# use to the "dummy" cache that doesn't actually cache -- it just implements
# the cache interface without doing anything.
#
# If you need caching, see the docs to choose a caching backend:
# https://docs.djangoproject.com/en/1.8/ref/settings/#caches
CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
    }
}

###############
# Application #
###############

#
# Back-end
#

# The type of workers to use.
# http://docs.gunicorn.org/en/latest/settings.html#worker-class
# Default: 'gevent'
NSOT_WORKER_CLASS = 'gevent'

# Load application code before the worker processes are forked.
# http://docs.gunicorn.org/en/latest/settings.html#preload-app
# Default: False
NSOT_PRELOAD = False

# The maximum number of requests a worker will process before restarting.
# http://docs.gunicorn.org/en/latest/settings.html#max-requests
# Default: 0
NSOT_MAX_REQUESTS = 0

# The maximum jitter to add to the max_requests setting. The jitter causes the
# restart per worker to be randomized by randint(0, max_requests_jitter). This
# is intended to stagger worker restarts to avoid all workers restarting at the
# same time.
# http://docs.gunicorn.org/en/latest/settings.html#max-requests-jitter
# Default: 0
NSOT_MAX_REQUESTS_JITTER = 0

#
# User-configurable
#

# The address on which the application will listen.
# Default: 'localhost'
NSOT_HOST = 'localhost'

# The port on which the application will be accessed.
# Default: 8990
NSOT_PORT = 8990

# The number of gunicorn worker processes for handling requests.
# Default: 4
NSOT_NUM_WORKERS = 4

# Timeout in seconds before gunicorn workers are killed/restarted.
# Default: 30
NSOT_WORKER_TIMEOUT = 30

# If True, serve static files directly from the app.
# Default: True
SERVE_STATIC_FILES = True

############
# Security #
############

# Header to check for Authenticated Email. This is intended for use behind an
# authenticating reverse proxy.
USER_AUTH_HEADER = 'X-NSoT-Email'

# The age, in seconds, until an AuthToken granted by the API will expire.
# Default: 600
AUTH_TOKEN_EXPIRY = 600  # 10 minutes

# A list of strings representing the host/domain names that this Django site
# can serve. This is a security measure to prevent an attacker from poisoning
# caches and triggering password reset emails with links to malicious hosts by
# submitting requests with a fake HTTP Host header, which is possible even
# under many seemingly-safe web server configurations.
# https://docs.djangoproject.com/en/1.8/ref/settings/#allowed-hosts
ALLOWED_HOSTS = ['*']

# The name of the cookie to use for the CSRF authentication token.
# Default: 'csrftoken'
CSRF_COOKIE_NAME = '_xsrf'

# Coming in Django 1.9
# The name of the request header used for CSRF authentication.
# https://github.com/django/django/commit/668d53c
# Default: 'X-CSRFToken'
# CSRF_HEADER_NAME = 'X-XSRFToken'

# Whether to create new users as superusers. If True, when new users are created
# they will be given superuser permissions. If False, new users will not be
# given superuser and it will be expected that permissions will be defined to
# control admin privileges. This only applies to users automatically created
# when authenticated using the "auth header" method, which is the default.
# Default: True
NSOT_NEW_USERS_AS_SUPERUSER = True

############
# Guardian #
############

# This setting is to silence any warnings raised by Guardian due to the usage
# of a custom auth backend implementation as opposed to
# guardian.backends.ObjectPermissionBackend
SILENCED_SYSTEM_CHECKS = ['guardian.W001']

# The Guardian anonymous user is different to the Django Anonymous user.
# The Django Anonymous user does not have an entry in the database,
# however the Guardian anonymous user does.
ANONYMOUS_USER_NAME = 'anonymous@service.local'

################
# Static files #
################

# (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.8/howto/static-files/

# URL to use when referring to static files located in STATIC_ROOT.
# Default: '/static/'
STATIC_URL = '/static/'

# This setting defines the additional locations the staticfiles app will
# traverse if the FileSystemFinder finder is enabled, e.g. if you use the
# collectstatic or findstatic management command or use the static file serving
# view.
# Default: $BASE_DIR/static
STATICFILES_DIRS = (
    os.path.join(BASE_DIR, 'static'),
)

# The absolute path to the directory where collectstatic will collect static
# files for deployment.
# Default: $BASE_DIR/staticfiles'
STATIC_ROOT = os.path.realpath(os.path.join(BASE_DIR, 'staticfiles'))

###########
# Swagger #
###########

SWAGGER_SETTINGS = {
    'exclude_namespaces': ['index'],
}

###########
# Logging #
###########
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': "[%(asctime)s] [%(process)d] [%(levelname)s] %(message)s",
            'datefmt': "%Y-%m-%d %H:%M:%S %z"
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'stream': sys.stdout,
            'formatter': 'verbose',
        },
    },
    'loggers': {
        'django.request': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'ERROR',
        },
        'nsot': {
            'handlers': ['console'],
            'level': 'INFO',
        },
        'rest_framework': {
            'handlers': ['console'],
            'level': 'ERROR',
        },
        'nsot_server': {
            'handlers': ['console'],
            'level': 'INFO',
        },
    }
}

##############
# Attributes #
##############

# Acceptable regex pattern for naming Attribute objects.
ATTRIBUTE_NAME = re.compile(r"^[a-z][a-z0-9_]*$")

###########
# Devices #
###########

# Acceptable regex pattern for naming Device objects.
DEVICE_NAME = re.compile(r"^([A-Za-z0-9][A-Za-z0-9\-]{0,61}[A-Za-z0-9]|[A-Za-z0-9])$")

##############
# Interfaces #
##############

# The default format for displaying MAC addresses. This defaults to
# ":"-separated and expanded (e.g. '00:00:00:00:00:00')
MACADDRESS_DEFAULT_DIALECT = 'macaddress.mac_linux'

# The default speed in Mbps for newly device interfaces if not otherwise
# specified.
INTERFACE_DEFAULT_SPEED = 1000  # In Mbps (e.g. 1Gbps)

# Default MAC address ('00:00:00:00:00:00')
INTERFACE_DEFAULT_MAC = eui.EUI(0, dialect=macaddress.default_dialect())

# These are mappings to the formal integer types from SNMP IF-MIB::ifType. The
# types listed here are the most commonly found in the wild.
#
# *IF YOU ARE GOING TO MODIFY THIS*: This MUST be a list of 2-tuples of
# (iftype, name), where iftype is the unique ID for the IANA ifType SNMP MIB,
# and name is whatever name your little heart desires, but hopefully matches
# the legit description according to the MIB.
#
# Ref: https://www.iana.org/assignments/ianaiftype-mib/ianaiftype-mib
INTERFACE_TYPE_CHOICES = (
    (6, 'ethernet'),       # for all ethernet-like interfaces
    (1, 'other'),          # none of the following
    (135, 'l2vlan'),       # Layer 2 Virtual LAN using 802.1Q
    (136, 'l3vlan'),       # Layer 3 Virtual LAN using IP
    (161, 'lag'),          # IEEE 802.3ad Link Aggregat
    (24, 'loopback'),      # softwareLoopback
    (150, 'mpls'),         # MPLS Tunnel Virtual Interface
    (53, 'prop_virtual'),  # proprietary virtual/internal
    (131, 'tunnel'),       # Encapsulation interface
)

INTERFACE_DEFAULT_TYPE = 6  # ethernet

############
# Networks #
############

# How long is an interconnect? (aka a point-to-point link)
NETWORK_INTERCONNECT_PREFIXES = (31, 127)

# CIDR prefix lengths for host addresses
HOST_PREFIXES = (32, 128)

# Valid IP versions
IP_VERSIONS = ('4', '6')

# Whether to compress IPv6 for display purposes, for example:
# - Exploded (default): 2620:0100:6000:0000:0000:0000:0000:0000/40
# - Compressed: 2620:100:6000::/40
# Default: True
NSOT_COMPRESS_IPV6 = True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import pytest
# Allow everything in there to access the DB
pytestmark = pytest.mark.django_db

from django.db import IntegrityError
from django.db.models import ProtectedError
from django.core.exceptions import (ValidationError as DjangoValidationError,
                                    MultipleObjectsReturned)
import logging

from nsot import exc, models

from .fixtures import admin_user, user, site, transactional_db


def test_creation(site):
    """Test explicit value creation."""
    attr = models.Attribute.objects.create(
        resource_name='Device',
        site=site, name='test_attribute'
    )
    dev = models.Device.objects.create(
        hostname='foo-bar1', site=site
    )

    # Explicitly create a Value without providing site_id
    val = models.Value.objects.create(
        obj=dev, attribute=attr, value='foo'
    )

    # Value site should match attribute
    assert val.site == attr.site

    # Device attributes should match a simple dict
    dev.clean_attributes()
    dev.save()
    assert dev.get_attributes() == {'test_attribute': 'foo'}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.db.backends.sqlite3.base import DatabaseWrapper
from django.db import models
from django.utils.datastructures import DictWrapper
from django_extensions.db.fields.json import JSONField
from macaddress.fields import MACAddressField as BaseMACAddressField
import ipaddress
import logging
from smart_selects.db_fields import ChainedForeignKey
import types

from . import exc


__all__ = (
    'BinaryIPAddressField', 'ChainedForeignKey', 'JSONField', 'MACAddressField'
)


log = logging.getLogger(__name__)


if not hasattr(DatabaseWrapper, 'get_new_connection_is_patched'):
    """
    Monkey-patch SQLite3 driver to handle text as bytes.

    Credit: http://stackoverflow.com/a/28794677/194311
    """
    _get_new_connection = DatabaseWrapper.get_new_connection

    def _get_new_connection_tolerant(self, conn_params):
        conn = _get_new_connection(self, conn_params)
        conn.text_factory = bytes
        return conn

    DatabaseWrapper.get_new_connection = types.MethodType(
        _get_new_connection_tolerant, None, DatabaseWrapper
    )
    DatabaseWrapper.get_new_connection_is_patched = True


class BinaryIPAddressField(models.Field):
    """IP Address field that stores values as varbinary."""
    __metaclass__ = models.SubfieldBase

    def __init__(self, *args, **kwargs):
        super(BinaryIPAddressField, self).__init__(*args, **kwargs)
        self.editable = True

    def db_type(self, connection):
        engine = connection.settings_dict['ENGINE']

        # Use the native 'inet' type for Postgres.
        if 'postgres' in engine:
            return 'inet'

        # Or 'varbinary' for everyone else.
        data = DictWrapper(self.__dict__, connection.ops.quote_name, "qn_")
        return 'varbinary(%(max_length)s)' % data

    def to_python(self, value):
        """DB -> Python."""
        if not value:
            return value

        try:
            obj = ipaddress.ip_address(unicode(value))
        except ValueError:
            obj = ipaddress.ip_address(bytes(value))

        # Display IPv6 as compressed or not? This is a no-op vor IPv4.
        if settings.NSOT_COMPRESS_IPV6:
            return obj.compressed

        return obj.exploded

    def get_db_prep_value(self, value, connection, prepared=False):
        """Python -> DB."""
        # To account for null defaults when performing migrations
        if value is None:
            return None

        engine = connection.settings_dict['ENGINE']

        # Send the value as-is to Postgres.
        if 'postgres' in engine:
            return value

        # Or packed binary for everyone else.
        return ipaddress.ip_address(value).packed


class MACAddressField(BaseMACAddressField):
    """
    Subclass of base field to raise a DRF ValidationError.

    DRF handles Django's default ValidationError, but this is so that we can
    always expect the DRF version, for better consistency in debugging and
    testing.
    """
    def to_python(self, value):
        try:
            return super(MACAddressField, self).to_python(value)
        except exc.DjangoValidationError as err:
            raise exc.ValidationError(err.message)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import smart_selects.db_fields
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0011_auto_20150930_1557'),
    ]

    operations = [
        migrations.AlterField(
            model_name='interface',
            name='description',
            field=models.CharField(default='', help_text='A brief yet helpful description.', max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='interface',
            name='device',
            field=models.ForeignKey(related_name='interfaces', verbose_name='Device', to='nsot.Device', help_text='Unique ID of the connected Device.'),
        ),
        migrations.AlterField(
            model_name='interface',
            name='parent',
            field=smart_selects.db_fields.ChainedForeignKey(chained_model_field='device', related_name='children', on_delete=django.db.models.deletion.PROTECT, default=None, auto_choose=True, to='nsot.Interface', chained_field='device', blank=True, help_text='Unique ID of the parent Interface.', null=True, verbose_name='Parent'),
        ),
        migrations.AlterField(
            model_name='interface',
            name='type',
            field=models.IntegerField(default=6, help_text="If not provided, defaults to 'ethernet'.", db_index=True, verbose_name='Interface Type', choices=[(6, b'ethernet'), (1, b'other'), (135, b'l2vlan'), (136, b'l3vlan'), (161, b'lag'), (24, b'loopback'), (150, b'mpls'), (53, b'prop_virtual'), (131, b'tunnel')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields.json


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0035_fix_interface_name_slug'),
    ]

    operations = [
        migrations.CreateModel(
            name='Protocol',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('_attributes_cache', django_extensions.db.fields.json.JSONField(help_text='Local cache of attributes. (Internal use only)', blank=True)),
                ('auth_string', models.CharField(default='', help_text='Authentication string (such as MD5 sum)', max_length=255, blank=True)),
                ('description', models.CharField(default='', help_text='Description for this Protocol', max_length=255, blank=True)),
                ('circuit', models.ForeignKey(related_name='protocols', blank=True, to='nsot.Circuit', help_text='Circuit that this protocol is running over.', null=True)),
                ('device', models.ForeignKey(related_name='protocols', to='nsot.Device', help_text='Device that this protocol is running on')),
                ('interface', models.ForeignKey(related_name='protocols', blank=True, to='nsot.Interface', help_text='Interface this protocol is running on. Either interface or circuit must be populated.', null=True)),
                ('site', models.ForeignKey(related_name='protocols', on_delete=django.db.models.deletion.PROTECT, blank=True, to='nsot.Site', help_text="Unique ID of the Site this Protocol is under. If not set, this be inherited off of the device's site", null=True, verbose_name='Site')),
            ],
            options={
                'ordering': ('device',),
            },
        ),
        migrations.CreateModel(
            name='ProtocolType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(help_text='Name of this type of protocol (e.g. OSPF, BGP, etc.)', unique=True, max_length=16, db_index=True)),
                ('description', models.CharField(default='', help_text='A description for this ProtocolType', max_length=255, blank=True)),
            ],
        ),
        migrations.AlterField(
            model_name='attribute',
            name='resource_name',
            field=models.CharField(help_text='The name of the Resource to which this Attribute is bound.', max_length=20, verbose_name='Resource Name', db_index=True, choices=[('Device', 'Device'), ('Interface', 'Interface'), ('Protocol', 'Protocol'), ('Network', 'Network'), ('Circuit', 'Circuit')]),
        ),
        migrations.AlterField(
            model_name='change',
            name='resource_name',
            field=models.CharField(help_text='The name of the Resource for this Change.', max_length=20, verbose_name='Resource Type', db_index=True, choices=[('Protocol', 'Protocol'), ('Network', 'Network'), ('Attribute', 'Attribute'), ('Site', 'Site'), ('Interface', 'Interface'), ('Circuit', 'Circuit'), ('Device', 'Device')]),
        ),
        migrations.AlterField(
            model_name='network',
            name='is_ip',
            field=models.BooleanField(default=False, help_text='Whether the Network is a host address or not.', db_index=True, editable=False),
        ),
        migrations.AlterField(
            model_name='value',
            name='resource_name',
            field=models.CharField(help_text='The name of the Resource type to which the Value is bound.', max_length=20, verbose_name='Resource Type', db_index=True, choices=[('Protocol', 'Protocol'), ('Network', 'Network'), ('Attribute', 'Attribute'), ('Site', 'Site'), ('Interface', 'Interface'), ('Circuit', 'Circuit'), ('Device', 'Device')]),
        ),
        migrations.AddField(
            model_name='protocoltype',
            name='required_attributes',
            field=models.ManyToManyField(help_text='All Attributes which are required by this ProtocolType. If a Protocol of this type is saved and is missing one of these attributes, a ValidationError will be raised.', related_name='protocol_types', to='nsot.Attribute', db_index=True),
        ),
        migrations.AddField(
            model_name='protocol',
            name='type',
            field=models.ForeignKey(related_name='protocols', to='nsot.ProtocolType', help_text='The type of this Protocol'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0036_add_protocol'),
    ]

    operations = [
        migrations.AddField(
            model_name='protocoltype',
            name='site',
            field=models.ForeignKey(related_name='protocol_types', on_delete=django.db.models.deletion.PROTECT, default=1, verbose_name='Site', to='nsot.Site', help_text='Unique ID of the Site this ProtocolType is under.'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='change',
            name='resource_name',
            field=models.CharField(help_text='The name of the Resource for this Change.', max_length=20, verbose_name='Resource Type', db_index=True, choices=[('Protocol', 'Protocol'), ('Network', 'Network'), ('ProtocolType', 'ProtocolType'), ('Attribute', 'Attribute'), ('Site', 'Site'), ('Interface', 'Interface'), ('Circuit', 'Circuit'), ('Device', 'Device')]),
        ),
        migrations.AlterField(
            model_name='protocol',
            name='auth_string',
            field=models.CharField(default='', help_text='Authentication string (such as MD5 sum)', max_length=255, verbose_name='Auth String', blank=True),
        ),
        migrations.AlterField(
            model_name='protocoltype',
            name='name',
            field=models.CharField(help_text='Name of this type of protocol (e.g. OSPF, BGP, etc.)', max_length=16, db_index=True),
        ),
        migrations.AlterField(
            model_name='value',
            name='resource_name',
            field=models.CharField(help_text='The name of the Resource type to which the Value is bound.', max_length=20, verbose_name='Resource Type', db_index=True, choices=[('Device', 'Device'), ('Interface', 'Interface'), ('Protocol', 'Protocol'), ('Network', 'Network'), ('Circuit', 'Circuit')]),
        ),
        migrations.AlterUniqueTogether(
            name='protocoltype',
            unique_together=set([('site', 'name')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import pytest

# Allow everything in there to access the DB
pytestmark = pytest.mark.django_db

import copy
from django.core.urlresolvers import reverse
import json
import logging
from rest_framework import status

from .fixtures import live_server, client, user, site
from .util import (
    assert_created, assert_error, assert_success, assert_deleted, load_json,
    Client, load, filter_networks, mkcidr, get_result
)


log = logging.getLogger(__name__)


def test_creation(live_server, user, site):
    """Test creation of Networks."""
    admin_client = Client(live_server, 'admin')
    user_client = Client(live_server, 'user')
    cidr = '10.0.0.0/24'

    # URIs
    site_uri = site.list_uri()
    attr_uri = site.list_uri('attribute')
    net_uri = site.list_uri('network')

    admin_client.create(attr_uri, resource_name='Network', name='attr1')

    # Invalid permissions
    assert_error(
        user_client.create(
            net_uri, cidr=cidr, attributes={'attr1': 'foo'}
        ),
        status.HTTP_403_FORBIDDEN
    )

    # Missing required field (cidr)
    assert_error(
        admin_client.create(net_uri, attributes={'attr1': 'foo'}),
        status.HTTP_400_BAD_REQUEST
    )

    # Null cidr
    assert_error(
        admin_client.create(net_uri, cidr=None),
        status.HTTP_400_BAD_REQUEST
    )

    # Verify Successful Creation
    net_resp = admin_client.create(
        net_uri, cidr=cidr, attributes={'attr1': 'foo'},
        state='reserved',
    )
    net = get_result(net_resp)
    net_obj_uri = site.detail_uri('network', id=net['id'])

    assert_created(net_resp, net_obj_uri)

    # Verify Successful get of all Networks
    payload = get_result(net_resp)
    expected = [payload]

    assert_success(admin_client.get(net_uri), expected)

    # Verify Successful get of single Network
    assert_success(admin_client.get(net_obj_uri), net)

    # Verify successful get of single Network by natural_key
    net_natural_uri = site.detail_uri('network', id=cidr)
    assert_success(admin_client.get(net_natural_uri), net)

    # Test creation by network_address, prefix_length
    params = {'network_address': '10.8.0.0', 'prefix_length': 16}
    net2_resp = admin_client.create(net_uri, **params)
    net2 = get_result(net2_resp)
    net2_obj_uri = site.detail_uri('network', id=net2['id'])
    assert_created(net2_resp, net2_obj_uri)

    # Delete it and then re-recreate it w/ the original payload
    assert_deleted(admin_client.delete(net2_obj_uri))
    net2a_resp = admin_client.create(net_uri, **params)
    net2a = get_result(net2a_resp)
    net2a_obj_uri = site.detail_uri('network', id=net2a['id'])
    assert_created(net2a_resp, net2a_obj_uri)


def test_bulk_operations(site, client):
    """Test creating/updating multiple Networks at once."""
    # URIs
    attr_uri = site.list_uri('attribute')
    net_uri = site.list_uri('network')

    # Successfully create a collection of Networks
    collection = [
        {'cidr': '1.1.1.0/24'},
        {'cidr': '2.2.2.0/24'},
        {'cidr': '3.3.3.0/24'},
    ]
    collection_response = client.post(
        net_uri,
        data=json.dumps(collection)
    )
    assert_created(collection_response, None)

    # Successfully get all created Networks
    output = collection_response.json()
    payload = get_result(output)

    assert_success(client.get(net_uri), payload)

    # Test bulk update to add attributes to each Network
    client.create(attr_uri, resource_name='Network', name='vlan')
    updated = copy.deepcopy(payload)
    for item in updated:
        item['attributes'] = {'vlan': '300'}
    updated_resp = client.put(net_uri, data=json.dumps(updated))
    expected = updated_resp.json()

    assert updated == expected


def test_filters(site, client):
    """Test cidr/address/prefix/attribute filters for Networks."""

    # URIs
    attr_uri = site.list_uri('attribute')
    net_uri = site.list_uri('network')

    # Pre-load the Attributes
    client.post(attr_uri, data=load('attributes.json'))

    # Populate the Network objects and retreive them for testing.
    client.post(net_uri, data=load('networks.json'))
    net_resp = client.retrieve(net_uri)
    networks = get_result(net_resp)

    # Test lookup by cidr
    wanted = ['10.0.0.0/8']
    expected = filter_networks(networks, wanted)
    assert_success(
        client.retrieve(net_uri, cidr='10.0.0.0/8'),
        expected
    )

    # Test lookup by attributes
    wanted = ['192.168.0.0/16', '172.16.0.0/12']
    expected = filter_networks(networks, wanted)
    assert_success(
        client.retrieve(net_uri, attributes='foo=baz'),
        expected
    )

    # Test lookup with multiple attributes
    wanted = ['172.16.0.0/12']
    expected = filter_networks(networks, wanted)
    assert_success(
        client.retrieve(net_uri, attributes=['foo=baz', 'cluster=lax']),
        expected
    )

    # Test lookup by network_address
    wanted = ['169.254.0.0/16']
    expected = filter_networks(networks, wanted)
    assert_success(
        client.retrieve(net_uri, network_address='169.254.0.0'),
        expected
    )

    # Test lookup by prefix_length
    wanted = ['192.168.0.0/16', '169.254.0.0/16']
    expected = filter_networks(networks, wanted)
    assert_success(
        client.retrieve(net_uri, prefix_length=16),
        expected
    )

    # Test lookup by network_address + prefix_length
    wanted = ['10.0.0.0/8']
    expected = filter_networks(networks, wanted)
    assert_success(
        client.retrieve(net_uri, network_address='10.0.0.0', prefix_length=8),
        expected
    )

    # Test include_ips=True, include_networks=False
    wanted = ['192.168.0.1/32']
    expected = filter_networks(networks, wanted)
    assert_success(
        client.retrieve(net_uri, include_ips=True, include_networks=False),
        expected
    )

    # Test include_ips=False, include_networks=True
    wanted = ['192.168.0.0/16', '10.0.0.0/8', '172.16.0.0/12',
              '169.254.0.0/16', '192.168.0.0/24', '192.168.0.0/25']
    expected = filter_networks(networks, wanted)
    assert_success(
        client.retrieve(net_uri, include_ips=False, include_networks=True),
        expected
    )

    # Test include_ips=True, include_networks=True
    wanted = ['192.168.0.0/16', '10.0.0.0/8', '172.16.0.0/12',
              '169.254.0.0/16', '192.168.0.1/32', '192.168.0.0/24',
              '192.168.0.0/25']
    expected = filter_networks(networks, wanted)
    assert_success(
        client.retrieve(net_uri, include_ips=True, include_networks=True),
        expected
    )

    # Test include_ips=False, include_networks=False
    wanted = []
    expected = filter_networks(networks, wanted)
    assert_success(
        client.retrieve(net_uri, include_ips=False, include_networks=False),
        expected
    )

    # Test lookup by ip_version (against ipv6)
    ipv6_resp = client.create(net_uri, cidr='2401:d:d0e::/64')
    ipv6 = get_result(ipv6_resp)
    wanted = ['2401:d:d0e::/64']
    expected = [ipv6]
    assert_success(
        client.retrieve(net_uri, ip_version='6'),
        expected
    )


def test_set_queries(client, site):
    """Test set queries for Networks."""

    # URIs
    attr_uri = site.list_uri('attribute')
    net_uri = site.list_uri('network')
    query_uri = site.query_uri('network')

    # Pre-load the attributes
    client.post(attr_uri, data=load('attributes.json'))

    # Populate the network objects and retreive them for testing.
    client.post(net_uri, data=load('networks.json'))
    net_resp = client.retrieve(net_uri)
    networks = get_result(net_resp)

    # INTERSECTION: foo=bar
    wanted = ['10.0.0.0/8', '169.254.0.0/16']
    expected = filter_networks(networks, wanted)
    assert_success(
        client.retrieve(query_uri, query='foo=bar'),
        expected
    )

    # INTERSECTION: foo=bar owner=jathan
    wanted = ['169.254.0.0/16']
    expected = filter_networks(networks, wanted)
    assert_success(
        client.retrieve(query_uri, query='foo=bar owner=jathan'),
        expected
    )

    # DIFFERENCE: -owner=gary, networks only
    wanted = ['192.168.0.0/16', '169.254.0.0/16', '192.168.0.0/24',
              '192.168.0.0/25']
    expected = filter_networks(networks, wanted)
    assert_success(
        client.retrieve(query_uri, query='-owner=gary', include_ips=False),
        expected
    )

    # UNION: cluster +foo=baz
    wanted = ['192.168.0.0/16', '172.16.0.0/12']
    expected = filter_networks(networks, wanted)
    assert_success(
        client.retrieve(query_uri, query='cluster +foo=baz'),
        expected
    )

    # Single IP result.
    wanted = ['192.168.0.1/32']
    expected = filter_networks(networks, wanted)
    assert_success(
        client.retrieve(query_uri, query='vlan=300'),
        expected
    )

    # UNIQUE: hostname=foo-bar1 vlan=300
    wanted = ['192.168.0.1/32']
    expected = filter_networks(networks, wanted)
    assert_success(
        client.retrieve(query_uri, query='hostname=foo-bar1 vlan=300', unique=True),
        expected
    )

    # ERROR: not unique
    assert_error(
        client.retrieve(query_uri, query='cluster +foo=baz', unique=True),
        status.HTTP_400_BAD_REQUEST
    )

    # ERROR: no results
    assert_error(
        client.retrieve(query_uri, query='cluster owner=bob', unique=True),
        status.HTTP_400_BAD_REQUEST
    )

    # ERROR: bad query
    assert_error(
        client.retrieve(query_uri, query='fake=bad'),
        status.HTTP_400_BAD_REQUEST
    )


def test_update(live_server, user, site):
    """Test updating Networks by pk and natural_key."""
    admin_client = Client(live_server, 'admin')
    user_client = Client(live_server, 'user')
    cidr = '10.0.0.0/24'

    # URIs
    site_uri = site.list_uri()
    attr_uri = site.list_uri('attribute')
    net_uri = site.list_uri('network')

    admin_client.create(attr_uri, resource_name='Network', name='attr1')
    net_resp = admin_client.create(
        net_uri, cidr=cidr, attributes={'attr1': 'foo'},
        state='reserved'
    )

    # Extract the Network object so that we can play w/ it during update tests.
    net = get_result(net_resp)
    net_obj_uri = site.detail_uri('network', id=net['id'])

    # Empty attributes should only clear attributes. Change state to 'allocatd'
    params = {'attributes': {}, 'state': 'allocated'}
    net.update(params)

    assert_success(
        admin_client.update(net_obj_uri, **params),
        net
    )

    # Now put attributes back
    params['attributes'] = {'attr1': 'foo'}
    net.update(params)

    assert_success(
        admin_client.update(net_obj_uri, **params),
        net
    )

    # Invalid permissions
    assert_error(user_client.update(net_obj_uri), status.HTTP_403_FORBIDDEN)

    # Test update by natural_key by zeroing out the attribtues again.
    params = {'attributes': {}, 'state': 'orphaned'}
    net.update(params)

    net_natural_uri = site.detail_uri('network', id=cidr)
    assert_success(
        admin_client.update(net_natural_uri, **params),
        net
    )


def test_partial_update(site, client):
    """"Test PATCH operations to partially update a Network."""
    net_uri = site.list_uri('network')
    attr_uri = site.list_uri('attribute')
    cidr = '10.0.0.0/24'

    client.create(attr_uri, resource_name='Network', name='attr1')
    net_resp = client.create(
        net_uri, cidr=cidr, attributes={'attr1': 'foo'}
    )

    # Extract the Network object so that we can play w/ it during update tests.
    net = get_result(net_resp)
    net_pk_uri = site.detail_uri('network', id=net['id'])
    net_natural_uri = site.detail_uri('network', id=cidr)

    # Now PATCH it by providing *only* the state, which wouldn't be
    # possible in a PUT.
    params = {'state': 'reserved'}
    net.update(params)

    assert_success(
        client.partial_update(net_pk_uri, **params),
        net
    )

    # And just to make sure a PUT with the same payload fails...
    assert_error(
        client.update(net_pk_uri, **params),
        status.HTTP_400_BAD_REQUEST
    )


def test_deletion(site, client):
    """Test deletion of Networks."""
    net_uri = site.list_uri('network')
    attr_uri = site.list_uri('attribute')

    net1_resp = client.create(net_uri, cidr='10.0.0.0/24')
    net1 = get_result(net1_resp)
    net1_obj_uri = site.detail_uri('network', id=net1['id'])

    net2_resp = client.create(net_uri, cidr='10.0.0.1/32')
    net2 = get_result(net2_resp)
    net2_obj_uri = site.detail_uri('network', id=net2['id'])

    # Don't allow delete when there's an attached subnet/ip
    assert_error(client.delete(net1_obj_uri), status.HTTP_409_CONFLICT)

    # Delete the child Network
    client.delete(net2_obj_uri)

    # And safely delete the parent Network
    assert_deleted(client.delete(net1_obj_uri))

    # Create Network 3 and delete it by natural_key
    net3_resp = client.create(net_uri, cidr='10.0.0.0/8')
    net3 = get_result(net3_resp)
    net3_natural_uri = site.detail_uri('network', id=mkcidr(net3))
    assert_deleted(client.delete(net3_natural_uri))


def test_mptt_detail_routes(site, client):
    """Test detail routes for ancestor/children/descendants/root methods."""
    net_uri = site.list_uri('network')

    client.create(net_uri, cidr='10.0.0.0/8')
    client.create(net_uri, cidr='10.16.0.0/12')
    client.create(net_uri, cidr='10.16.0.0/14')
    client.create(net_uri, cidr='10.16.2.0/25')
    client.create(net_uri, cidr='10.16.2.1/32')
    client.create(net_uri, cidr='10.16.2.2/32')

    net_8_resp = client.retrieve(net_uri, cidr='10.0.0.0/8')
    net_8 = get_result(net_8_resp)[0]
    net_8_obj_uri = site.detail_uri('network', id=net_8['id'])

    net_12_resp = client.retrieve(net_uri, cidr='10.16.0.0/12')
    net_12 = get_result(net_12_resp)[0]
    net_12_obj_uri = site.detail_uri('network', id=net_12['id'])

    net_14_resp = client.retrieve(net_uri, cidr='10.16.0.0/14')
    net_14 = get_result(net_14_resp)[0]
    net_14_obj_uri = site.detail_uri('network', id=net_14['id'])

    net_25_resp = client.retrieve(net_uri, cidr='10.16.2.0/25')
    net_25 = get_result(net_25_resp)[0]
    net_25_obj_uri = site.detail_uri('network', id=net_25['id'])

    ip1_resp = client.retrieve(net_uri, cidr='10.16.2.1/32')
    ip1 = get_result(ip1_resp)[0]
    ip1_obj_uri = site.detail_uri('network', id=ip1['id'])

    ip2_resp = client.retrieve(net_uri, cidr='10.16.2.2/32')
    ip2 = get_result(ip2_resp)[0]
    ip2_obj_uri = site.detail_uri('network', id=ip2['id'])

    # ancestors
    expected = [net_8, net_12, net_14]
    uri = reverse('network-ancestors', args=(site.id, net_25['id']))
    natural_uri = reverse('network-ancestors', args=(site.id, mkcidr(net_25)))
    assert_success(client.retrieve(uri), expected)

    expected = [net_14, net_12, net_8]
    assert_success(client.retrieve(uri, ascending=True), expected)
    assert_success(client.retrieve(natural_uri, ascending=True), expected)

    # children
    uri = reverse('network-children', args=(site.id, net_25['id']))
    natural_uri = reverse('network-children', args=(site.id, mkcidr(net_25)))
    wanted = [ip1, ip2]
    expected = wanted
    assert_success(client.retrieve(uri), expected)
    assert_success(client.retrieve(natural_uri), expected)

    uri = reverse('network-children', args=(site.id, net_12['id']))
    natural_uri = reverse('network-children', args=(site.id, mkcidr(net_12)))
    wanted = [net_14]
    expected = wanted
    assert_success(client.retrieve(uri), expected)
    assert_success(client.retrieve(natural_uri), expected)

    # descendants (spelled correctly)
    uri = reverse('network-descendants', args=(site.id, net_8['id']))
    natural_uri = reverse('network-descendants', args=(site.id, mkcidr(net_8)))
    wanted = [net_12, net_14, net_25, ip1, ip2]
    expected = wanted
    assert_success(client.retrieve(uri), expected)
    assert_success(client.retrieve(natural_uri), expected)

    uri = reverse('network-descendants', args=(site.id, net_14['id']))
    natural_uri = reverse(
        'network-descendants', args=(site.id, mkcidr(net_14))
    )
    wanted = [net_25, ip1, ip2]
    expected = wanted
    assert_success(client.retrieve(uri), expected)
    assert_success(client.retrieve(natural_uri), expected)

    uri = reverse('network-descendants', args=(site.id, ip2['id']))
    natural_uri = reverse('network-descendants', args=(site.id, mkcidr(ip2)))
    expected = []
    assert_success(client.retrieve(uri), expected)
    assert_success(client.retrieve(natural_uri), expected)

    # descendents (spelled incorrectly) should send along a "Warning" header
    # TODO(jathan): This should be removed no earlier than v1.3 release.
    uri = reverse('network-descendents', args=(site.id, net_14['id']))
    expected = [net_25, ip1, ip2]
    response = client.retrieve(uri)
    assert_success(response, expected)
    assert response.headers.get('Warning') is not None

    # parent
    uri = reverse('network-parent', args=(site.id, ip2['id']))
    natural_uri = reverse('network-parent', args=(site.id, mkcidr(ip2)))
    assert_success(client.retrieve(uri), net_25)
    assert_success(client.retrieve(natural_uri), net_25)

    # root
    uri = reverse('network-root', args=(site.id, ip1['id']))
    natural_uri = reverse('network-root', args=(site.id, mkcidr(ip1)))
    assert_success(client.retrieve(uri), net_8)
    assert_success(client.retrieve(natural_uri), net_8)

    uri = reverse('network-root', args=(site.id, net_8['id']))
    natural_uri = reverse('network-root', args=(site.id, mkcidr(net_8)))
    assert_error(client.retrieve(uri), status.HTTP_404_NOT_FOUND)
    assert_error(client.retrieve(natural_uri), status.HTTP_404_NOT_FOUND)

    # siblings
    uri = reverse('network-siblings', args=(site.id, ip1['id']))
    natural_uri = reverse('network-siblings', args=(site.id, mkcidr(ip1)))
    wanted = [ip2]
    expected = wanted
    assert_success(client.retrieve(uri), expected)
    assert_success(client.retrieve(natural_uri), expected)

    wanted = [ip1, ip2]
    expected = wanted
    assert_success(client.retrieve(uri, include_self=True), expected)
    assert_success(client.retrieve(natural_uri, include_self=True), expected)


def test_get_next_detail_routes(site, client):
    """Test the detail routes for getting next available networks/addresses."""
    net_uri = site.list_uri('network')

    client.create(net_uri, cidr='10.16.2.0/25')
    client.create(net_uri, cidr='10.16.2.8/29')
    client.create(net_uri, cidr='10.16.2.1/32')
    client.create(net_uri, cidr='10.16.2.2/32')
    client.create(net_uri, cidr='10.16.2.4/32')
    client.create(net_uri, cidr='10.16.2.17/32')

    net_25_resp = client.retrieve(net_uri, cidr='10.16.2.0/25')
    net_25 = get_result(net_25_resp)[0]
    net_25_obj_uri = site.detail_uri('network', id=net_25['id'])

    net_29_resp = client.retrieve(net_uri, cidr='10.16.2.8/29')
    net_29 = get_result(net_29_resp)[0]
    net_29_obj_uri = site.detail_uri('network', id=net_29['id'])

    ip1_resp = client.retrieve(net_uri, cidr='10.16.2.1/32')
    ip1 = get_result(ip1_resp)[0]
    ip1_obj_uri = site.detail_uri('network', id=ip1['id'])

    ip2_resp = client.retrieve(net_uri, cidr='10.16.2.2/32')
    ip2 = get_result(ip2_resp)[0]
    ip2_obj_uri = site.detail_uri('network', id=ip2['id'])

    ip3_resp = client.retrieve(net_uri, cidr='10.16.2.2/32')
    ip3 = get_result(ip3_resp)[0]
    ip3_obj_uri = site.detail_uri('network', id=ip3['id'])

    #
    # next_network
    #
    uri = reverse('network-next-network', args=(site.id, net_25['id']))
    natural_uri = reverse(
        'network-next-network', args=(site.id, mkcidr(net_25))
    )

    # A single /28
    expected = [u'10.16.2.32/28']
    assert_success(client.retrieve(uri, prefix_length=28), expected)
    assert_success(client.retrieve(natural_uri, prefix_length=28), expected)

    # 3x remaining /27 in the /25
    networks = [u'10.16.2.32/27', u'10.16.2.64/27', u'10.16.2.96/27']
    assert_success(
        client.retrieve(uri, prefix_length=27, num=3),
        networks
    )
    assert_success(
        client.retrieve(natural_uri, prefix_length=27, num=3),
        networks
    )

    # Missing/invalid prefix_length
    ## by pk
    assert_error(client.retrieve(uri), status.HTTP_400_BAD_REQUEST)
    assert_error(client.retrieve(uri, prefix_length='ralph'), status.HTTP_400_BAD_REQUEST)
    assert_error(client.retrieve(uri, prefix_length=14), status.HTTP_400_BAD_REQUEST)
    assert_error(client.retrieve(uri, prefix_length=65), status.HTTP_400_BAD_REQUEST)

    ## by natural_key
    assert_error(client.retrieve(natural_uri), status.HTTP_400_BAD_REQUEST)
    assert_error(
        client.retrieve(natural_uri, prefix_length='ralph'),
        status.HTTP_400_BAD_REQUEST
    )
    assert_error(
        client.retrieve(natural_uri, prefix_length=14),
        status.HTTP_400_BAD_REQUEST
    )
    assert_error(
        client.retrieve(natural_uri, prefix_length=65), status.HTTP_400_BAD_REQUEST
    )

    # Invalid num
    assert_error(
        client.retrieve(uri, prefix_length=28, num='potato'),
        status.HTTP_400_BAD_REQUEST
    )

    #
    # next_address
    #
    uri = reverse('network-next-address', args=(site.id, net_25['id']))
    natural_uri = reverse(
        'network-next-address', args=(site.id, mkcidr(net_25))
    )

    # A single /32
    assert_success(client.retrieve(uri), [u'10.16.2.3/32'])
    assert_success(client.retrieve(natural_uri), [u'10.16.2.3/32'])

    # 3x /32
    addresses = [u'10.16.2.3/32', u'10.16.2.5/32', u'10.16.2.6/32']
    assert_success(client.retrieve(uri, num=3), addresses)
    assert_success(client.retrieve(natural_uri, num=3), addresses)

    # Invalid num is all we can really test for.
    assert_error(
        client.retrieve(uri, num='potato'),
        status.HTTP_400_BAD_REQUEST
    )
    assert_error(
        client.retrieve(natural_uri, num='potato'),
        status.HTTP_400_BAD_REQUEST
    )


def test_next_network_allocation(site, client):
    net_uri = site.list_uri('network')

    client.create(net_uri, cidr='10.1.2.0/24')

    net_24_resp = client.retrieve(net_uri, cidr='10.1.2.0/24')
    net_24 = get_result(net_24_resp)[0]
    net_24_obj_uri = site.detail_uri('network', id=net_24['id'])

    uri = reverse('network-next-network', args=(site.id, net_24['id']))

    client.post(uri, params={u'prefix_length': u'32'})
    assert_success(client.retrieve(uri, prefix_length=32), [u'10.1.2.2/32'])

    client.post(uri, params={u'prefix_length': u'32', u'reserve': u'True'})

    uri = reverse('network-reserved', args=(site.id,))
    assert get_result(client.retrieve(uri))[0]['network_address'] == u'10.1.2.2'


def test_next_address_allocation(site, client):
    net_uri = site.list_uri('network')

    client.create(net_uri, cidr='10.1.2.0/24')

    net_24_resp = client.retrieve(net_uri, cidr='10.1.2.0/24')
    net_24 = get_result(net_24_resp)[0]
    net_24_obj_uri = site.detail_uri('network', id=net_24['id'])

    uri = reverse('network-next-address', args=(site.id, net_24['id']))

    client.post(uri)
    assert_success(client.retrieve(uri, prefix_length=32), [u'10.1.2.2/32'])

    client.post(uri, params={u'reserve': u'True'})

    uri = reverse('network-reserved', args=(site.id,))
    assert get_result(client.retrieve(uri))[0]['network_address'] == u'10.1.2.2'


def test_reservation_list_route(site, client):
    """Test the list route for getting reserved networks/addresses."""
    net_uri = site.list_uri('network')
    res_uri = reverse('network-reserved', args=(site.id,))

    net_resp = client.create(net_uri, cidr='192.168.3.0/24', state='reserved')
    net = get_result(net_resp)

    # Fetch the reserved networks and make sure they match up.
    networks = [net]
    expected = networks
    assert_success(client.retrieve(res_uri), expected)


def test_closest_parent_detail_route(site, client):
    """
    Test the detail route for looking up the closest parent for a
    Network.

    GET /api/sites/1/networks/10.250.0.1/32/closest_parent/
    """
    net_uri = site.list_uri('network')

    # Create networks
    root_resp = client.create(net_uri, cidr='10.250.0.0/16')
    root = get_result(root_resp)
    parent_resp = client.create(net_uri, cidr='10.250.0.0/24')
    parent = get_result(parent_resp)

    # To make sure that a /25 doesn't return as closest parent. See: issue #209
    client.create(net_uri, cidr='10.250.0.0/25')

    # Closest parent for non-existent /32 should be ``parent``
    closest_uri = reverse(
        'network-closest-parent', args=(site.id, '10.250.0.185/32')
    )
    expected = parent
    assert_success(client.retrieve(closest_uri), expected)

    # Matching ip with shorter prefix_length should 404
    assert_error(
        client.retrieve(closest_uri, prefix_length=27),
        status.HTTP_404_NOT_FOUND
    )

    # Invalid prefix_length should 400
    assert_error(
        client.retrieve(closest_uri, prefix_length='shoe'),
        status.HTTP_400_BAD_REQUEST
    )

    # Invalid cidr should 400
    bad_closest_uri = reverse(
        'network-closest-parent', args=(site.id, 1)
    )
    assert_error(client.retrieve(bad_closest_uri), status.HTTP_400_BAD_REQUEST)

    # Alpha cidr should 404
    abc_closest_uri = reverse(
        'network-closest-parent', args=(site.id, 'bogus')
    )
    assert_error(client.retrieve(abc_closest_uri), status.HTTP_404_NOT_FOUND)

    # If a closest parent isn't found it should 404
    no_closest_uri = reverse(
        'network-closest-parent', args=(site.id, '1.0.0.1/32')
    )
    assert_error(client.retrieve(no_closest_uri), status.HTTP_404_NOT_FOUND)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0031_populate_circuit_name_slug'),
    ]

    operations = [
        migrations.AlterField(
            model_name='change',
            name='change_at',
            field=models.DateTimeField(help_text='The timestamp of this Change.', auto_now_add=True, db_index=True),
        ),
        migrations.AlterIndexTogether(
            name='change',
            index_together=set([('resource_name', 'resource_id'), ('resource_name', 'event')]),
        ),
    ]



"""
General purpose utilities for unit-testing.
"""

import json
import os


__all__ = ('load_json',)


def load_json(relpath):
    """
    Load JSON files relative to ``tests`` directory.

    Files are loaded from the 'data' directory. So for example for
    ``/path/to/data/devices/foo.json`` the ``relpath`` would be
    ``data/devices/foo.json``.

    :param relpath:
        Relative path to our directory's "data" dir
    """
    our_path = os.path.dirname(os.path.abspath(__file__))
    filepath = os.path.join(our_path, relpath)
    with open(filepath, 'rb') as f:
        return json.load(f)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0029_auto__add_circuit'),
    ]

    operations = [
        migrations.AddField(
            model_name='circuit',
            name='name_slug',
            field=models.CharField(db_index=True, editable=False, max_length=255, help_text='Slugified version of the name field, used for the natural key', null=True, unique=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0006_auto_20150810_1947'),
    ]

    operations = [
        migrations.RenameField(
            model_name='interface',
            old_name='mac',
            new_name='mac_address',
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import pytest

# Allow everything in there to access the DB
pytestmark = pytest.mark.django_db

import copy
from django.core.urlresolvers import reverse
import json
import logging
from rest_framework import status

from .fixtures import live_server, client, user, site, user_client
from .util import (
    assert_created, assert_error, assert_success, assert_deleted, load_json,
    Client, load, filter_circuits, get_result
)


log = logging.getLogger(__name__)


def test_creation(site, client):
    """Test basic creation of a Circuit."""
    cir_uri = site.list_uri('circuit')
    dev_uri = site.list_uri('device')
    ifc_uri = site.list_uri('interface')
    net_uri = site.list_uri('network')

    # Devices
    dev_a_resp = client.create(dev_uri, hostname='foo-bar1')
    dev_a = get_result(dev_a_resp)
    dev_z_resp = client.create(dev_uri, hostname='foo-bar2')
    dev_z = get_result(dev_z_resp)

    # Parent network for interface assignments
    net_resp = client.create(net_uri, cidr='10.32.0.0/24')
    net = get_result(net_resp)

    # Interfaces
    if_a_resp = client.create(
        ifc_uri, device=dev_a['id'], name='eth0', addresses=['10.32.0.1/32']
    )
    if_a = get_result(if_a_resp)
    if_z_resp = client.create(
        ifc_uri, device=dev_z['id'], name='eth0', addresses=['10.32.0.2/32']
    )
    if_z = get_result(if_z_resp)

    # Missing required field (endpoint_a, endoint_z)
    assert_error(
        client.create(cir_uri, attributes={'attr1': 'foo'}),
        status.HTTP_400_BAD_REQUEST
    )

    # Null name is bad
    assert_error(
        client.create(
            cir_uri, endpoint_a=if_a['id'], endpoint_z=if_z['id'], name=None
        ),
        status.HTTP_400_BAD_REQUEST
    )

    # Verify successful creation
    cir_resp = client.create(
        cir_uri, endpoint_a=if_a['id'], endpoint_z=if_z['id']
    )
    cir = get_result(cir_resp)
    cir_obj_uri = site.detail_uri('circuit', id=cir['id'])

    assert_created(cir_resp, cir_obj_uri)

    # Try to create another circuit with the same interfaces
    assert_error(
        client.create(
            cir_uri, endpoint_a=if_a['id'], endpoint_z=if_z['id']
        ),
        status.HTTP_400_BAD_REQUEST
    )

    # Try to create another circuit with interfaces A/Z inverted.
    assert_error(
        client.create(
            cir_uri, endpoint_a=if_z['id'], endpoint_z=if_a['id']
        ),
        status.HTTP_400_BAD_REQUEST
    )

    # Create a circuit referencing interfaces with natural keys
    ae1_a = get_result(client.create(ifc_uri, device=dev_a['id'], name='ae1'))
    ae1_z = get_result(client.create(ifc_uri, device=dev_z['id'], name='ae1'))

    cir2_resp = client.create(
        cir_uri, endpoint_a=ae1_a['name_slug'], endpoint_z=ae1_z['name_slug']
    )
    cir2 = get_result(cir2_resp)
    cir2_obj_uri = site.detail_uri('circuit', id=cir2['id'])

    assert_created(cir2_resp, cir2_obj_uri)

    # Verify successful get of single Circuit
    assert_success(client.get(cir_obj_uri), cir)

    # Verify successful get of single Circuit by natural key
    natural_key = cir['name']
    cir_natural_uri = site.detail_uri('circuit', id=natural_key)
    assert_success(client.get(cir_natural_uri), cir)

    # Verify successful retrieval of all Circuits
    circuits = [cir, cir2]
    expected = circuits
    assert_success(client.get(cir_uri), expected)


def test_bulk_operations(site, client):
    """Test creating/updating multiple Circuits at once."""
    cir_uri = site.list_uri('circuit')
    dev_uri = site.list_uri('device')
    ifc_uri = site.list_uri('interface')

    # Devices
    dev_a_resp = client.create(dev_uri, hostname='foo-bar1')
    dev_a = get_result(dev_a_resp)
    dev_z_resp = client.create(dev_uri, hostname='foo-bar2')
    dev_z = get_result(dev_z_resp)

    # Interfaces (2-pairs of A/Z)
    # 1st pair
    if_a1_resp = client.create(ifc_uri, device=dev_a['id'], name='eth0')
    if_a1 = get_result(if_a1_resp)
    if_z1_resp = client.create(ifc_uri, device=dev_z['id'], name='eth0')
    if_z1 = get_result(if_z1_resp)
    # 2nd pair
    if_a2_resp = client.create(ifc_uri, device=dev_a['id'], name='eth1')
    if_a2 = get_result(if_a2_resp)
    if_z2_resp = client.create(ifc_uri, device=dev_z['id'], name='eth1')
    if_z2 = get_result(if_z2_resp)

    # Successfully create a collection of Circuits
    collection = [
        {'endpoint_a': if_a1['id'], 'endpoint_z': if_z1['id']},
        {'endpoint_a': if_a2['id'], 'endpoint_z': if_z2['id']},
    ]
    collection_response = client.post(
        cir_uri,
        data=json.dumps(collection)
    )
    assert_created(collection_response, None)

    # Successfully get all created Circuits
    output = collection_response.json()
    expected = get_result(output)

    assert_success(client.get(cir_uri), expected)

    # Test update of all created Circuit (name: foo => wtf)
    updated = copy.deepcopy(expected)
    for item in updated:
        item['name'] = item['name'].replace('foo', 'wtf')
        item['name_slug'] = item['name_slug'].replace('foo', 'wtf')
    updated_resp = client.put(cir_uri, data=json.dumps(updated))
    expected = updated_resp.json()

    assert updated == expected


def test_update(site, client):
    """Test update of an existing interface w/ an address."""
    cir_uri = site.list_uri('circuit')
    dev_uri = site.list_uri('device')
    ifc_uri = site.list_uri('interface')

    # Devices
    dev_a_resp = client.create(dev_uri, hostname='foo-bar1')
    dev_a = get_result(dev_a_resp)
    dev_z_resp = client.create(dev_uri, hostname='foo-bar2')
    dev_z = get_result(dev_z_resp)

    # Interfaces
    if_a_resp = client.create(ifc_uri, device=dev_a['id'], name='eth0')
    if_a = get_result(if_a_resp)
    if_z_resp = client.create(ifc_uri, device=dev_z['id'], name='eth0')
    if_z = get_result(if_z_resp)

    # Create basic circuit
    cir_resp = client.create(
        cir_uri, endpoint_a=if_a['id'], endpoint_z=if_z['id']
    )
    cir = get_result(cir_resp)
    cir_obj_uri = site.detail_uri('circuit', id=cir['id'])

    # Update circuit to use a different Z-side (using natural key).
    if_z2_resp = client.create(ifc_uri, device=dev_z['id'], name='eth1')
    if_z2 = get_result(if_z2_resp)

    payload = copy.deepcopy(cir)
    params = copy.deepcopy(payload)
    params['endpoint_z'] = if_z2['name_slug']
    payload.update(params)

    assert_success(
        client.update(cir_obj_uri, **params),
        payload
    )

    # Update circuit Z-side using ID (should still match previous payload)
    params['endpoint_z'] = if_z2['id']
    assert_success(
        client.update(cir_obj_uri, **params),
        payload
    )

    # Updating circuit to have no A-size should fail!
    params['endpoint_a'] = None
    assert_error(
        client.update(cir_obj_uri, **params),
        status.HTTP_400_BAD_REQUEST
    )
    params['endpoint_a'] = if_a['name_slug']  # Restore if_a.name_slug

    # Update circuit to have no Z-side.
    params['endpoint_z'] = None
    params['name'] = 'foo-bar1:eth0_None'
    payload.update(params)

    # Read-only computed parameters
    payload['name_slug'] = 'foo-bar1:eth0_None'

    assert_success(
        client.update(cir_obj_uri, **params),
        payload
    )

    # Check the circuit's name is foo-bar1:eth0_None
    expected_name = '%s:%s_%s' % (if_a['device_hostname'], if_a['name'], None)
    obj_resp = client.get(cir_obj_uri)
    obj = get_result(obj_resp)
    assert obj['name'] == expected_name

    # Restore circuit back to normal using natural_key
    params['endpoint_z'] = if_z['name_slug']
    params['name'] = 'foo-bar1:eth0_foo-bar2:eth0'
    payload.update(params)

    natural_key = obj['name']
    cir_natural_uri = site.detail_uri('circuit', id=natural_key)

    assert_success(
        client.update(cir_natural_uri, **params),
        payload
    )

    # Create another A-side interface on dev_a
    if_a2_resp = client.create(ifc_uri, device=dev_a['id'], name='eth1')
    if_a2 = get_result(if_a2_resp)

    # Create another circuit
    cir2_resp = client.create(
        cir_uri, endpoint_a=if_a2['id'], endpoint_z=if_z2['id']
    )
    cir2 = get_result(cir2_resp)
    cir2_obj_uri = site.detail_uri('circuit', id=cir2['id'])

    # Circuit name must be unique (try setting cir2 to cir's name)
    params = copy.deepcopy(cir2)
    params['name'] = cir['name']
    assert_error(
        client.update(cir2_obj_uri, **params),
        status.HTTP_400_BAD_REQUEST
    )


def test_partial_update(site, client):
    """Test PATCH operations to partially update a Circuit."""
    attr_uri = site.list_uri('attribute')
    cir_uri = site.list_uri('circuit')
    dev_uri = site.list_uri('device')
    ifc_uri = site.list_uri('interface')

    # Devices
    dev_a_resp = client.create(dev_uri, hostname='foo-bar1')
    dev_a = get_result(dev_a_resp)
    dev_z_resp = client.create(dev_uri, hostname='foo-bar2')
    dev_z = get_result(dev_z_resp)

    # Interfaces
    if_a_resp = client.create(ifc_uri, device=dev_a['id'], name='eth0')
    if_a = get_result(if_a_resp)
    if_z_resp = client.create(ifc_uri, device=dev_z['id'], name='eth0')
    if_z = get_result(if_z_resp)

    # Pre-load the Attributes
    client.post(attr_uri, data=load('attributes.json'))

    # Create basic circuit
    cir_resp = client.create(
        cir_uri, endpoint_a=if_a['id'], endpoint_z=if_z['id'],
        attributes={'cid': 'abc123', 'vendor': 'acme'}
    )
    cir = get_result(cir_resp)
    cir_obj_uri = site.detail_uri('circuit', id=cir['id'])

    # Assert that a partial update on PUT will fail
    params = {'name': 'mycircuit'}
    assert_error(
        client.update(cir_obj_uri, **params),
        status.HTTP_400_BAD_REQUEST
    )

    # Update only name
    payload = copy.deepcopy(cir)
    payload.update(params)
    payload['name_slug'] = 'mycircuit'

    assert_success(
        client.partial_update(cir_obj_uri, **params),
        payload
    )

    # Update only attributes
    params = {'attributes': {}}  # Nuke 'em
    payload.update(params)
    assert_success(
        client.partial_update(cir_obj_uri, **params),
        payload
    )

    # Update only endpoint_z (to null)
    params = {'endpoint_z': None}
    payload.update(params)
    assert_success(
        client.partial_update(cir_obj_uri, **params),
        payload
    )


def test_filters(site, client):
    """Test field filters for Interfaces."""
    attr_uri = site.list_uri('attribute')
    cir_uri = site.list_uri('circuit')
    dev_uri = site.list_uri('device')
    ifc_uri = site.list_uri('interface')

    # Devices
    dev_a_resp = client.create(dev_uri, hostname='foo-bar1')
    dev_a = get_result(dev_a_resp)
    dev_z_resp = client.create(dev_uri, hostname='foo-bar2')
    dev_z = get_result(dev_z_resp)

    # Interfaces (2-pairs of A/Z)
    # 1st pair
    if_a1_resp = client.create(ifc_uri, device=dev_a['id'], name='eth0')
    if_a1 = get_result(if_a1_resp)
    if_z1_resp = client.create(ifc_uri, device=dev_z['id'], name='eth0')
    if_z1 = get_result(if_z1_resp)
    # 2nd pair
    if_a2_resp = client.create(ifc_uri, device=dev_a['id'], name='eth1')
    if_a2 = get_result(if_a2_resp)
    if_z2_resp = client.create(ifc_uri, device=dev_z['id'], name='eth1')
    if_z2 = get_result(if_z2_resp)

    # Pre-load the Attributes
    client.post(attr_uri, data=load('attributes.json'))

    # Create the circuits
    client.create(
        cir_uri, endpoint_a=if_a1['id'], endpoint_z=if_z1['id'],
        attributes={'cid': 'abc123', 'vendor': 'acme'}
    )
    client.create(
        cir_uri, endpoint_a=if_a2['id'], endpoint_z=if_z2['id'],
        attributes={'cid': 'abc246', 'vendor': 'acme'}
    )

    # Populate the Circuit objects and retreive them for testing.
    circuits_resp = client.get(cir_uri)
    circuits = get_result(circuits_resp)
    cir1, cir2 = circuits

    # Test filter by name
    wanted = [cir1]
    expected = filter_circuits(circuits, wanted)
    assert_success(
        client.retrieve(cir_uri, name='foo-bar1:eth0_foo-bar2:eth0'),
        expected
    )

    # Test filter by endpoint_a
    wanted = [cir2]
    expected = filter_circuits(circuits, wanted)
    # by ID
    assert_success(
        client.retrieve(cir_uri, endpoint_a=if_a2['id']),
        expected
    )
    # by natural key
    assert_success(
        client.retrieve(cir_uri, endpoint_a=if_a2['name_slug']),
        expected
    )

    # Test filter by endpoint_z
    wanted = [cir1]
    expected = filter_circuits(circuits, wanted)
    # by ID
    assert_success(
        client.retrieve(cir_uri, endpoint_z=if_z1['id']),
        expected
    )
    # by natural key
    assert_success(
        client.retrieve(cir_uri, endpoint_z=if_z1['name_slug']),
        expected
    )

    # Test filter by attributes (cidr=abc246)
    wanted = [cir2]
    expected = filter_circuits(circuits, wanted)
    assert_success(
        client.retrieve(cir_uri, attributes=['cid=abc246']),
        expected
    )

    # Test filter by attributes (vendor=acme)
    wanted = [cir1, cir2]
    expected = filter_circuits(circuits, wanted)
    assert_success(
        client.retrieve(cir_uri, attributes=['vendor=acme']),
        expected
    )


def test_set_queries(client, site):
    """Test set queries for Interfaces."""
    # URIs
    cir_uri = site.list_uri('circuit')
    dev_uri = site.list_uri('device')
    attr_uri = site.list_uri('attribute')
    ifc_uri = site.list_uri('interface')
    query_uri = site.query_uri('circuit')

    # Pre-load the Attributes
    client.post(attr_uri, data=load('attributes.json'))

    # Devices
    dev_a_resp = client.create(dev_uri, hostname='foo-bar1')
    dev_a = get_result(dev_a_resp)
    dev_z_resp = client.create(dev_uri, hostname='foo-bar2')
    dev_z = get_result(dev_z_resp)

    # Interfaces (4-pairs of A/Z)
    # 1st pair
    if_a1_resp = client.create(ifc_uri, device=dev_a['id'], name='eth0')
    if_a1 = get_result(if_a1_resp)
    if_z1_resp = client.create(ifc_uri, device=dev_z['id'], name='eth0')
    if_z1 = get_result(if_z1_resp)
    # 2nd pair
    if_a2_resp = client.create(ifc_uri, device=dev_a['id'], name='eth1')
    if_a2 = get_result(if_a2_resp)
    if_z2_resp = client.create(ifc_uri, device=dev_z['id'], name='eth1')
    if_z2 = get_result(if_z2_resp)
    # 3rd pair
    if_a3_resp = client.create(ifc_uri, device=dev_a['id'], name='eth2')
    if_a3 = get_result(if_a3_resp)
    if_z3_resp = client.create(ifc_uri, device=dev_z['id'], name='eth2')
    if_z3 = get_result(if_z3_resp)
    # 3rd pair
    if_a4_resp = client.create(ifc_uri, device=dev_a['id'], name='eth3')
    if_a4 = get_result(if_a4_resp)
    if_z4_resp = client.create(ifc_uri, device=dev_z['id'], name='eth3')
    if_z4 = get_result(if_z4_resp)

    # Create 4x circuits
    client.create(
        cir_uri, endpoint_a=if_a1['id'], endpoint_z=if_z1['id'],
        attributes={'cid': 'abc123', 'vendor': 'acme'}
    )
    client.create(
        cir_uri, endpoint_a=if_a2['id'], endpoint_z=if_z2['id'],
        attributes={'cid': 'abc246', 'vendor': 'acme'}
    )
    client.create(
        cir_uri, endpoint_a=if_a3['id'], endpoint_z=if_z3['id'],
        attributes={'cid': 'xyz123', 'vendor': 'blamco'}
    )
    client.create(
        cir_uri, endpoint_a=if_a4['id'], endpoint_z=if_z4['id'],
        attributes={'cid': 'xyz246', 'vendor': 'jathcorp'}
    )

    # Populate the Interface objects and retreive them for testing.
    circuits_resp = client.get(cir_uri)
    circuits = get_result(circuits_resp)
    cir1, cir2, cir3, cir4 = circuits

    # INTERSECTION: vendor=acme
    wanted = [cir1, cir2]
    expected = filter_circuits(circuits, wanted)
    assert_success(
        client.retrieve(query_uri, query='vendor=acme'),
        expected
    )
    # INTERSECTION: cid=abc246 vendor=acme
    wanted = [cir2]
    expected = filter_circuits(circuits, wanted)
    assert_success(
        client.retrieve(query_uri, query='cid=abc246 vendor=acme'),
        expected
    )

    # DIFFERENCE: -vendor=acme
    wanted = [cir3, cir4]
    expected = filter_circuits(circuits, wanted)
    assert_success(
        client.retrieve(query_uri, query='-vendor=acme'),
        expected
    )

    # UNION: vendor=blamco +vendor=jathcorp
    wanted = [cir3, cir4]
    expected = filter_circuits(circuits, wanted)
    assert_success(
        client.retrieve(query_uri, query='vendor=blamco +vendor=jathcorp'),
        expected
    )

    # UNIQUE: cidr=abc123 vendor=acme
    wanted = [cir1]
    expected = filter_circuits(circuits, wanted)
    assert_success(
        client.retrieve(query_uri, query='cid=abc123 vendor=acme', unique=True),
        expected
    )

    # ERROR: not unique (vendor=acme)
    assert_error(
        client.retrieve(query_uri, query='vendor=acme', unique=True),
        status.HTTP_400_BAD_REQUEST
    )

    # ERROR: no result (vendor=bogus)
    assert_error(
        client.retrieve(query_uri, query='vendor=bogus', unique=True),
        status.HTTP_400_BAD_REQUEST
    )

    # ERROR: bad query
    assert_error(
        client.retrieve(query_uri, query='bacon=delicious'),
        status.HTTP_400_BAD_REQUEST
    )


def test_deletion(site, client):
    """Test deletion of Circuits."""
    cir_uri = site.list_uri('circuit')
    dev_uri = site.list_uri('device')
    ifc_uri = site.list_uri('interface')
    net_uri = site.list_uri('network')

    # Devices
    dev_a_resp = client.create(dev_uri, hostname='foo-bar1')
    dev_a = get_result(dev_a_resp)
    dev_z_resp = client.create(dev_uri, hostname='foo-bar2')
    dev_z = get_result(dev_z_resp)

    # Interfaces
    if_a_resp = client.create(ifc_uri, device=dev_a['id'], name='eth0')
    if_a = get_result(if_a_resp)
    if_z_resp = client.create(ifc_uri, device=dev_z['id'], name='eth0')
    if_z = get_result(if_z_resp)

    # Circuit
    cir_resp = client.create(
        cir_uri, endpoint_a=if_a['id'], endpoint_z=if_z['id']
    )
    cir = get_result(cir_resp)
    cir_obj_uri = site.detail_uri('circuit', id=cir['id'])

    # Don't allow Interface delete when member of a Circuit
    if_a_uri = site.detail_uri('interface', id=if_a['id'])
    assert_error(client.delete(if_a_uri), status.HTTP_409_CONFLICT)

    # Delete the Circuit
    assert_deleted(client.delete(cir_obj_uri))

    # And safely delete the A-side interface
    assert_deleted(client.delete(if_a_uri))

    # Delete by natural key (re-create interface and circuit first)
    if_a2_resp = client.create(ifc_uri, device=dev_a['id'], name='eth0')
    if_a2 = get_result(if_a2_resp)
    cir2_resp = client.create(
        cir_uri, endpoint_a=if_a2['id'], endpoint_z=if_z['id']
    )
    cir2 = get_result(cir2_resp)
    cir2_natural_uri = site.detail_uri('circuit', id=cir2['name'])
    assert_deleted(client.delete(cir2_natural_uri))


def test_detail_routes(site, client):
    """Test detail routes for Circuit objects."""
    cir_uri = site.list_uri('circuit')
    dev_uri = site.list_uri('device')
    ifc_uri = site.list_uri('interface')
    net_uri = site.list_uri('network')

    # Devices
    dev_a_resp = client.create(dev_uri, hostname='foo-bar1')
    dev_a = get_result(dev_a_resp)
    dev_z_resp = client.create(dev_uri, hostname='foo-bar2')
    dev_z = get_result(dev_z_resp)

    # Parent network for interface assignments
    net_resp = client.create(net_uri, cidr='10.32.0.0/24')
    net = get_result(net_resp)

    # Interfaces
    if_a_resp = client.create(
        ifc_uri, device=dev_a['id'], name='eth0', addresses=['10.32.0.1/32']
    )
    if_a = get_result(if_a_resp)
    if_z_resp = client.create(
        ifc_uri, device=dev_z['id'], name='eth0', addresses=['10.32.0.2/32']
    )
    if_z = get_result(if_z_resp)

    # Circuit
    cir_resp = client.create(
        cir_uri, endpoint_a=if_a['id'], endpoint_z=if_z['id']
    )
    cir = get_result(cir_resp)
    cir_obj_uri = site.detail_uri('circuit', id=cir['id'])

    # Verify Circuit.addresses
    addresses_resp = client.retrieve(net_uri, prefix_length=32)
    expected = get_result(addresses_resp)
    # pk
    addresses_uri = reverse('circuit-addresses', args=(site.id, cir['id']))
    assert_success(client.retrieve(addresses_uri), expected)
    # natural_key
    addresses_uri = reverse('circuit-addresses', args=(site.id, cir['name']))
    assert_success(client.retrieve(addresses_uri), expected)

    # Verify Circuit.interfaces
    interfaces_resp = client.retrieve(ifc_uri)
    expected = get_result(interfaces_resp)
    # pk
    interfaces_uri = reverse('circuit-interfaces', args=(site.id, cir['id']))
    assert_success(client.retrieve(interfaces_uri), expected)
    # natural_key
    interfaces_uri = reverse('circuit-interfaces', args=(site.id, cir['name']))
    assert_success(client.retrieve(interfaces_uri), expected)

    # Verify Circuit.devices
    devices_resp = client.retrieve(dev_uri)
    expected = get_result(devices_resp)
    # pk
    devices_uri = reverse('circuit-devices', args=(site.id, cir['id']))
    assert_success(client.retrieve(devices_uri), expected)
    # natural_key
    devices_uri = reverse('circuit-devices', args=(site.id, cir['name']))
    assert_success(client.retrieve(devices_uri), expected)

    # Verify Device.circuits
    dev_circuits_uri = reverse('device-circuits', args=(site.id, dev_a['id']))
    expected = [cir]
    assert_success(client.retrieve(dev_circuits_uri), expected)

    # Verify Interface.circuit
    ifc_circuit_uri = reverse('interface-circuit', args=(site.id, if_a['id']))
    expected = cir
    assert_success(client.retrieve(ifc_circuit_uri), expected)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


def update_interface_device_hostname(apps, schema_editor):
    """Update all interfaces with hostname from associated device"""
    Device = apps.get_model('nsot', 'Device')
    for dev in Device.objects.iterator():
        dev.interfaces.update(device_hostname=dev.hostname)


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0027_interface_device_hostname'),
    ]

    operations = [

        # Save all devices
        migrations.RunPython(update_interface_device_hostname),

    ]



from django.contrib.auth.models import Group
import pytest
from pytest_django.fixtures import  django_user_model, transactional_db
import logging

from nsot import models


log = logging.getLogger(__name__)


@pytest.fixture
def user(django_user_model):
    """Create and return a non-admin user."""
    user = django_user_model.objects.create(email='user@localhost')
    return user


@pytest.fixture
def admin_user(django_user_model):
    """Create and return an admin user."""
    user = django_user_model.objects.create(
        email='admin@localhost', is_superuser=True, is_staff=True
    )
    return user


@pytest.fixture
def site():
    """Create and return a Site object."""
    site = models.Site.objects.create(
        name='Test Site', description='This is a Test Site.'
    )
    return site


@pytest.fixture
def device(site):
    """Create and return a Device object bound to ``site``."""
    device = models.Device.objects.create(site=site, hostname='foo-bar1')
    return device


@pytest.fixture
def circuit(site):
    """Create and return a Circuit object bound to ``site``."""
    device_a = models.Device.objects.create(site=site, hostname='foo-bar1')
    device_z = models.Device.objects.create(site=site, hostname='foo-bar2')

    # Create a network for interface assignments
    network = models.Network.objects.create(
        cidr='10.32.0.0/24', site=site,
    )

    # Create A/Z-side interfaces
    iface_a = models.Interface.objects.create(
        device=device_a, name='eth0', addresses=['10.32.0.1/32']
    )
    iface_z = models.Interface.objects.create(
        device=device_z, name='eth0', addresses=['10.32.0.2/32']
    )

    # Create the circuit
    circuit = models.Circuit.objects.create(
        endpoint_a=iface_a, endpoint_z=iface_z
    )
    return circuit


@pytest.fixture
def test_group():
    """Create and return a Group object."""
    test_group = Group.objects.create(name='test_group')
    return test_group



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import nsot.fields
import nsot.util.core
import django.db.models.deletion
from django.conf import settings
import django_extensions.db.fields.json


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0025_value_site'),
    ]

    operations = [
        migrations.AlterField(
            model_name='assignment',
            name='address',
            field=models.ForeignKey(related_name='assignments', to='nsot.Network', help_text='Network to which this assignment is bound.'),
        ),
        migrations.AlterField(
            model_name='assignment',
            name='interface',
            field=models.ForeignKey(related_name='assignments', to='nsot.Interface', help_text='Interface to which this assignment is bound.'),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='constraints',
            field=django_extensions.db.fields.json.JSONField(help_text='Dictionary of Attribute constraints.', verbose_name='Constraints', blank=True),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='description',
            field=models.CharField(default='', help_text='A helpful description of the Attribute.', max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='display',
            field=models.BooleanField(default=False, help_text='Whether the Attribute should be be displayed by default in UIs. If required is set, this is also set.'),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='multi',
            field=models.BooleanField(default=False, help_text='Whether the Attribute should be treated as a list type.'),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='name',
            field=models.CharField(help_text='The name of the Attribute.', max_length=64, db_index=True),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='required',
            field=models.BooleanField(default=False, help_text='Whether the Attribute should be required.'),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='resource_name',
            field=models.CharField(help_text='The name of the Resource to which this Attribute is bound.', max_length=20, verbose_name='Resource Name', db_index=True, choices=[('Device', 'Device'), ('Interface', 'Interface'), ('Network', 'Network')]),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='site',
            field=models.ForeignKey(related_name='attributes', on_delete=django.db.models.deletion.PROTECT, verbose_name='Site', to='nsot.Site', help_text='Unique ID of the Site this Attribute is under.'),
        ),
        migrations.AlterField(
            model_name='change',
            name='_resource',
            field=django_extensions.db.fields.json.JSONField(help_text='Local cache of the changed Resource. (Internal use only)', verbose_name='Resource', blank=True),
        ),
        migrations.AlterField(
            model_name='change',
            name='change_at',
            field=models.DateTimeField(help_text='The timestamp of this Change.', auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='change',
            name='event',
            field=models.CharField(help_text='The type of event this Change represents.', max_length=10, choices=[('Create', 'Create'), ('Update', 'Update'), ('Delete', 'Delete')]),
        ),
        migrations.AlterField(
            model_name='change',
            name='resource_id',
            field=models.IntegerField(help_text='The unique ID of the Resource for this Change.', verbose_name='Resource ID'),
        ),
        migrations.AlterField(
            model_name='change',
            name='resource_name',
            field=models.CharField(help_text='The name of the Resource for this Change.', max_length=20, verbose_name='Resource Type', db_index=True, choices=[('Device', 'Device'), ('Attribute', 'Attribute'), ('Interface', 'Interface'), ('Site', 'Site'), ('Network', 'Network')]),
        ),
        migrations.AlterField(
            model_name='change',
            name='site',
            field=models.ForeignKey(related_name='changes', verbose_name='Site', to='nsot.Site', help_text='Unique ID of the Site this Change is under.'),
        ),
        migrations.AlterField(
            model_name='change',
            name='user',
            field=models.ForeignKey(related_name='changes', to=settings.AUTH_USER_MODEL, help_text='The User that initiated this Change.'),
        ),
        migrations.AlterField(
            model_name='device',
            name='_attributes_cache',
            field=django_extensions.db.fields.json.JSONField(help_text='Local cache of attributes. (Internal use only)', blank=True),
        ),
        migrations.AlterField(
            model_name='device',
            name='hostname',
            field=models.CharField(help_text='The hostname of the Device.', max_length=255, db_index=True),
        ),
        migrations.AlterField(
            model_name='device',
            name='site',
            field=models.ForeignKey(related_name='devices', on_delete=django.db.models.deletion.PROTECT, verbose_name='Site', to='nsot.Site', help_text='Unique ID of the Site this Device is under.'),
        ),
        migrations.AlterField(
            model_name='interface',
            name='_attributes_cache',
            field=django_extensions.db.fields.json.JSONField(help_text='Local cache of attributes. (Internal use only)', blank=True),
        ),
        migrations.AlterField(
            model_name='interface',
            name='addresses',
            field=models.ManyToManyField(help_text='Network addresses assigned to this Interface', related_name='addresses', through='nsot.Assignment', to='nsot.Network', db_index=True),
        ),
        migrations.AlterField(
            model_name='interface',
            name='site',
            field=models.ForeignKey(related_name='interfaces', on_delete=django.db.models.deletion.PROTECT, to='nsot.Site', help_text='Unique ID of the Site this Interface is under.'),
        ),
        migrations.AlterField(
            model_name='interface',
            name='speed',
            field=models.IntegerField(default=1000, help_text='Integer of Mbps of interface (e.g. 20000 for 20 Gbps). If not provided, defaults to 1000.', db_index=True, blank=True),
        ),
        migrations.AlterField(
            model_name='network',
            name='_attributes_cache',
            field=django_extensions.db.fields.json.JSONField(help_text='Local cache of attributes. (Internal use only)', blank=True),
        ),
        migrations.AlterField(
            model_name='network',
            name='broadcast_address',
            field=nsot.fields.BinaryIPAddressField(help_text='The broadcast address for the Network. (Internal use only)', max_length=16, db_index=True),
        ),
        migrations.AlterField(
            model_name='network',
            name='is_ip',
            field=models.BooleanField(default=False, help_text='Whether the Network is a host address or not.', db_index=True),
        ),
        migrations.AlterField(
            model_name='network',
            name='network_address',
            field=nsot.fields.BinaryIPAddressField(help_text='The network address for the Network. The network address and the prefix length together uniquely define a network.', max_length=16, verbose_name='Network Address', db_index=True),
        ),
        migrations.AlterField(
            model_name='network',
            name='parent',
            field=models.ForeignKey(related_name='children', on_delete=django.db.models.deletion.PROTECT, default=None, blank=True, to='nsot.Network', help_text='The parent Network of the Network.', null=True),
        ),
        migrations.AlterField(
            model_name='network',
            name='prefix_length',
            field=models.IntegerField(help_text='Length of the Network prefix, in bits.', verbose_name='Prefix Length', db_index=True),
        ),
        migrations.AlterField(
            model_name='network',
            name='site',
            field=models.ForeignKey(related_name='networks', on_delete=django.db.models.deletion.PROTECT, verbose_name='Site', to='nsot.Site', help_text='Unique ID of the Site this Network is under.'),
        ),
        migrations.AlterField(
            model_name='network',
            name='state',
            field=models.CharField(default='allocated', help_text='The allocation state of the Network.', max_length=20, db_index=True, choices=[('allocated', 'Allocated'), ('assigned', 'Assigned'), ('orphaned', 'Orphaned'), ('reserved', 'Reserved')]),
        ),
        migrations.AlterField(
            model_name='site',
            name='description',
            field=models.TextField(default='', help_text='A helpful description for the Site.', blank=True),
        ),
        migrations.AlterField(
            model_name='site',
            name='name',
            field=models.CharField(help_text='The name of the Site.', unique=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='user',
            name='secret_key',
            field=models.CharField(default=nsot.util.core.generate_secret_key, help_text="The user's secret_key used for API authentication.", max_length=44),
        ),
        migrations.AlterField(
            model_name='value',
            name='attribute',
            field=models.ForeignKey(related_name='values', on_delete=django.db.models.deletion.PROTECT, to='nsot.Attribute', help_text='The Attribute to which this Value is assigned.'),
        ),
        migrations.AlterField(
            model_name='value',
            name='name',
            field=models.CharField(help_text='The name of the Attribute to which the Value is bound. (Internal use only)', max_length=64, verbose_name='Name', blank=True),
        ),
        migrations.AlterField(
            model_name='value',
            name='resource_id',
            field=models.IntegerField(help_text='The unique ID of the Resource to which the Value is bound.', verbose_name='Resource ID'),
        ),
        migrations.AlterField(
            model_name='value',
            name='resource_name',
            field=models.CharField(help_text='The name of the Resource type to which the Value is bound.', max_length=20, verbose_name='Resource Type', db_index=True, choices=[('Device', 'Device'), ('Attribute', 'Attribute'), ('Interface', 'Interface'), ('Site', 'Site'), ('Network', 'Network')]),
        ),
        migrations.AlterField(
            model_name='value',
            name='site',
            field=models.ForeignKey(related_name='values', on_delete=django.db.models.deletion.PROTECT, verbose_name='Site', to='nsot.Site', help_text='Unique ID of the Site this Value is under.'),
        ),
        migrations.AlterField(
            model_name='value',
            name='value',
            field=models.CharField(help_text='The Attribute value.', max_length=255, db_index=True, blank=True),
        ),
    ]
